@@ -33,6 +33,7 @@ void CL_Changing_f(void);
 void CL_Reconnect_f(void);
 void CL_Connect_f(void);
 void CL_Rcon_f(void);
+void CL_Packet_f(void);
 void CL_CheckForResend(void);
 
 cvar_t *rcon_client_password;
@@ -568,7 +569,8 @@ CL_InitLocal(void)
 	cl_vwep = Cvar_Get("cl_vwep", "1", CVAR_ARCHIVE);
 
 #ifdef USE_CURL
-	cl_http_proxy = Cvar_Get("cl_http_proxy", "", 0);
+	cl_http_verifypeer = Cvar_Get("cl_http_verifypeer", "1", CVAR_ARCHIVE);
+	cl_http_proxy = Cvar_Get("cl_http_proxy", "", CVAR_ARCHIVE);
 	cl_http_filelists = Cvar_Get("cl_http_filelists", "1", 0);
 	cl_http_downloads = Cvar_Get("cl_http_downloads", "1", CVAR_ARCHIVE);
 	cl_http_max_connections = Cvar_Get("cl_http_max_connections", "4", 0);
@@ -597,6 +599,7 @@ CL_InitLocal(void)
 	Cmd_AddCommand("reconnect", CL_Reconnect_f);
 
 	Cmd_AddCommand("rcon", CL_Rcon_f);
+	Cmd_AddCommand("packet", CL_Packet_f);
 
 	Cmd_AddCommand("setenv", CL_Setenv_f);
 
@@ -977,4 +980,6 @@ CL_Shutdown(void)
 	S_Shutdown();
 	IN_Shutdown();
 	VID_Shutdown();
+
+	Mods_NamesFinish();
 }
