@@ -158,10 +158,6 @@ CL_ClipMoveToEntities(vec3_t start, vec3_t mins, vec3_t maxs,
 				*tr = trace;
 			}
 		}
-		else if (trace.startsolid)
-		{
-			tr->startsolid = true;
-		}
 	}
 }
 
@@ -231,6 +227,7 @@ CL_PredictMovement(void)
 	pmove_t pm;
 	int i;
 	int step;
+	vec3_t tmp;
 
 	if (cls.state != ca_active)
 	{
@@ -295,10 +292,13 @@ CL_PredictMovement(void)
 		VectorCopy(pm.s.origin, cl.predicted_origins[frame]);
 	}
 
+	// step is used for movement prediction on stairs
+	// (so moving up/down stairs is smooth)
 	step = pm.s.origin[2] - (int)(cl.predicted_origin[2] * 8);
+	VectorCopy(pm.s.velocity, tmp);
 
-	if (((step > 126 && step < 130))
-		&& !VectorCompare((float *)pm.s.velocity, vec3_origin)
+	if (((step > 62 && step < 66) || (step > 94 && step < 98) || (step > 126 && step < 130))
+		&& !VectorCompare(tmp, vec3_origin)
 		&& (pm.s.pm_flags & PMF_ON_GROUND))
 	{
 		cl.predicted_step = step * 0.125f;
