@@ -421,12 +421,7 @@ void CL_Widowbeamout (cl_sustain_t *self);
 void CL_Nukeblast (cl_sustain_t *self);
 void CL_WidowSplash (vec3_t org);
 
-int CL_ParseEntityBits (unsigned *bits);
-void CL_ParseDelta (entity_state_t *from, entity_state_t *to, int number, int bits);
-void CL_ParseFrame (void);
-
 void CL_ParseTEnt (void);
-void CL_ParseConfigString (void);
 void CL_AddMuzzleFlash (void);
 void CL_AddMuzzleFlash2 (void);
 void SmokeAndFlash(vec3_t origin);
@@ -491,18 +486,25 @@ void CL_BaseMove (usercmd_t *cmd);
 
 void IN_CenterView (void);
 
+typedef enum
+{
+	LBL_SDL = 0,
+	LBL_XBOX,
+	LBL_PLAYSTATION,
+	LBL_SWITCH,
+	LBL_MAX_COUNT
+} gamepad_labels_t;
+
 float CL_KeyState (kbutton_t *key);
 char *Key_KeynumToString (int keynum);
+char *Key_KeynumToString_Joy (int key);
 
 void CL_WriteDemoMessage (void);
 void CL_Stop_f (void);
 void CL_Record_f (void);
 
-extern	char *svc_strings[256];
-
 void CL_ParseServerMessage (void);
 void CL_LoadClientinfo (clientinfo_t *ci, char *s);
-void SHOWNET(char *s);
 void CL_ParseClientinfo (int player);
 void CL_Download_f (void);
 
