@@ -35,6 +35,7 @@
  */
 
 #include <SDL3/SDL.h>
+#include <limits.h>
 
 #include "SDL3/SDL_gamepad.h"
 #include "SDL3/SDL_properties.h"
@@ -81,7 +82,7 @@ typedef enum
 // IN_Update() called at the beginning of a frame to the
 // actual movement functions called at a later time.
 static float mouse_x, mouse_y;
-static unsigned char sdl_back_button = SDL_GAMEPAD_BUTTON_BACK;
+static unsigned char joy_escbutton = SDL_GAMEPAD_BUTTON_START;
 static int joystick_left_x, joystick_left_y, joystick_right_x, joystick_right_y;
 static float gyro_yaw, gyro_pitch;
 static qboolean mlooking;
@@ -94,6 +95,12 @@ int sys_frame_time;
 // is pressed
 qboolean joy_altselector_pressed = false;
 
+// Gamepad labels' style (Xbox, Playstation, etc.) in use, normally set after detection
+gamepad_labels_t joy_current_lbls = LBL_SDL;
+
+// Using japanese style for confirm & cancel buttons on gamepad
+qboolean japanese_confirm = false;
+
 // Console Variables
 cvar_t *freelook;
 cvar_t *lookstrafe;
@@ -137,6 +144,12 @@ static int last_haptic_effect_size = HAPTIC_EFFECT_LIST_SIZE;
 static int last_haptic_effect_pos = 0;
 static haptic_effects_cache_t last_haptic_effect[HAPTIC_EFFECT_LIST_SIZE];
 
+// Gamepad labels' style (Xbox, Playstation, etc.) requested by user
+static cvar_t *joy_labels;
+
+// Gamepad style for confirm and cancel buttons (traditional or japanese)
+static cvar_t *joy_confirm;
+
 // Joystick sensitivity
 static cvar_t *joy_yawsensitivity;
 static cvar_t *joy_pitchsensitivity;
@@ -175,10 +188,19 @@ static float gyro_accum[3];
 static cvar_t *gyro_calibration_x;
 static cvar_t *gyro_calibration_y;
 static cvar_t *gyro_calibration_z;
-static unsigned int num_samples;
-#define NATIVE_SDL_GYRO	// uses SDL_CONTROLLERSENSORUPDATE to read gyro
 
-// To ignore SDL_JOYDEVICEADDED at game init. Allows for hot plugging of game controller afterwards.
+// If "SDL gyro" is not explicitly disabled, use SDL_EVENT_GAMEPAD_SENSOR_UPDATE to read gyro
+#ifndef NO_SDL_GYRO
+static unsigned int num_samples;
+#else // otherwise, gyro can be read as a "secondary joystick" exposed by dkms-hid-nintendo
+static unsigned int num_samples[3];
+static SDL_Joystick *imu_joystick = NULL;	// gyro "joystick"
+#define IMU_JOY_AXIS_GYRO_ROLL 3
+#define IMU_JOY_AXIS_GYRO_PITCH 4
+#define IMU_JOY_AXIS_GYRO_YAW 5
+#endif
+
+// To ignore SDL_EVENT_JOYSTICK_ADDED at game init. Allows for hot plugging of gamepad afterwards.
 static qboolean first_init = true;
 
 // Countdown of calls to IN_Update(), needed for controller init and gyro calibration
@@ -188,9 +210,10 @@ static unsigned short int updates_countdown = 30;
 static updates_countdown_reasons countdown_reason = REASON_CONTROLLERINIT;
 
 // Flick Stick
-#define FLICK_TIME 6		// number of frames it takes for a flick to execute
+#define FLICK_TIME 100		// time it takes for a flick to execute, in ms
 static float target_angle;	// angle to end up facing at the end of a flick
-static unsigned short int flick_progress = FLICK_TIME;
+static float flick_progress = 1.0f;	// from 0.0 to 1.0
+static int started_flick;	// time of flick start
 
 // Flick Stick's rotation input samples to smooth out
 #define MAX_SMOOTH_SAMPLES 8
@@ -225,10 +248,17 @@ IN_TranslateSDLtoQ2Key(unsigned int keysym)
 		case SDLK_BACKSPACE:
 			key = K_BACKSPACE;
 			break;
-		case SDLK_LGUI:
+#ifdef __APPLE__
 		case SDLK_RGUI:
-			key = K_COMMAND; // Win key
+		case SDLK_LGUI:
+			key = K_COMMAND;
 			break;
+#else
+		case SDLK_RGUI:
+		case SDLK_LGUI:
+			key = K_SUPER;
+			break;
+#endif
 		case SDLK_CAPSLOCK:
 			key = K_CAPSLOCK;
 			break;
@@ -384,8 +414,6 @@ IN_TranslateSDLtoQ2Key(unsigned int keysym)
 			key = K_KP_EQUALS;
 			break;
 
-		// TODO: K_SUPER ? Win Key is already K_COMMAND
-
 		case SDLK_APPLICATION:
 			key = K_COMPOSE;
 			break;
@@ -489,16 +517,108 @@ IN_TranslateScancodeToQ2Key(SDL_Scancode sc)
 static void IN_Controller_Init(qboolean notify_user);
 static void IN_Controller_Shutdown(qboolean notify_user);
 
+/*
+ * Sets the gamepad buttons' style of labels (SDL, Xbox, PS, Switch).
+ * They are only visible in the gamepad binding options.
+ * Traditional binding uses SDL style, no matter the gamepad.
+ */
+static void
+IN_GamepadLabels_Changed(void)
+{
+	const int requested = (int)joy_labels->value;
+	joy_labels->modified = false;
+	joy_current_lbls = LBL_SDL;
+
+	if (requested < 0 && controller) // try to autodetect...
+	{
+		switch (SDL_GetGamepadType(controller))
+		{
+			case SDL_GAMEPAD_TYPE_XBOX360:
+			case SDL_GAMEPAD_TYPE_XBOXONE:
+				joy_current_lbls = LBL_XBOX;
+				return;
+
+			case SDL_GAMEPAD_TYPE_PS3:
+			case SDL_GAMEPAD_TYPE_PS4:
+			case SDL_GAMEPAD_TYPE_PS5:
+				joy_current_lbls = LBL_PLAYSTATION;
+				return;
+
+			case SDL_GAMEPAD_TYPE_NINTENDO_SWITCH_PRO:
+			case SDL_GAMEPAD_TYPE_NINTENDO_SWITCH_JOYCON_PAIR:
+				joy_current_lbls = LBL_SWITCH;
+			default:
+				return;
+		}
+	}
+	else if (requested >= LBL_SDL && requested < LBL_MAX_COUNT)
+	{
+		joy_current_lbls = (gamepad_labels_t)requested;
+	}
+}
+
+/*
+ * Sets which gamepad button works as "confirm", and which
+ * works as "cancel", in menus.
+ */
+static void
+IN_GamepadConfirm_Changed(void)
+{
+	const int requested = (int)joy_confirm->value;
+	japanese_confirm = false;
+	joy_confirm->modified = false;
+
+	if (requested < 0 && controller) // try to autodetect...
+	{
+		switch (SDL_GetGamepadType(controller))
+		{
+			case SDL_GAMEPAD_TYPE_NINTENDO_SWITCH_PRO:
+			case SDL_GAMEPAD_TYPE_NINTENDO_SWITCH_JOYCON_PAIR:
+				japanese_confirm = true;
+			default:
+				return;
+		}
+	}
+	else if (requested == 1)
+	{
+		japanese_confirm = true;
+	}
+}
+
+static void
+IN_GyroMode_Changed(void)
+{
+	if (gyro_mode->value < 2)
+	{
+		gyro_active = false;
+	}
+	else
+	{
+		gyro_active = true;
+	}
+	gyro_mode->modified = false;
+}
+
+static void
+IN_VirtualKeyEvent(int keynum, qboolean *state_store, qboolean new_state)
+{
+	if (new_state != *state_store)
+	{
+		*state_store = new_state;
+		Key_Event(keynum, *state_store, true);
+	}
+}
+
 qboolean IN_NumpadIsOn()
 {
-    SDL_Keymod mod = SDL_GetModState();
+	SDL_Keymod mod = SDL_GetModState();
 
-    if ((mod & SDL_KMOD_NUM) == SDL_KMOD_NUM)
-    {
-        return true;
-    }
+	if ((mod & SDL_KMOD_NUM) == SDL_KMOD_NUM)
+	{
+		return true;
+	}
 
-    return false;
+	return false;
 }
 
 /* ------------------------------------------------------------------ */
@@ -517,6 +637,7 @@ IN_Update(void)
 
 	static qboolean left_trigger = false;
 	static qboolean right_trigger = false;
+	static qboolean left_stick[4] = {false, false, false, false};   // left, right, up, down virtual keys
 
 	static int consoleKeyCode = 0;
 
@@ -588,7 +709,7 @@ IN_Update(void)
 				/* workaround for AZERTY-keyboards, which don't have 1, 2, ..., 9, 0 in first row:
 				 * always map those physical keys (scancodes) to those keycodes anyway
 				 * see also https://bugzilla.libsdl.org/show_bug.cgi?id=3188 */
-				SDL_Scancode sc = event.key.keysym.scancode;
+				SDL_Scancode sc = event.key.scancode;
 
 				if (sc >= SDL_SCANCODE_1 && sc <= SDL_SCANCODE_0)
 				{
@@ -606,14 +727,14 @@ IN_Update(void)
 				}
 				else
 				{
-					SDL_Keycode kc = event.key.keysym.sym;
+					SDL_Keycode kc = event.key.key;
 					if(sc == SDL_SCANCODE_GRAVE && kc != '\'' && kc != '"')
 					{
 						// special case/hack: open the console with the "console key"
 						// (beneath Esc, left of 1, above Tab)
 						// but not if the keycode for this is a quote (like on Brazilian
 						// keyboards) - otherwise you couldn't type them in the console
-						if((event.key.keysym.mod & (SDL_KMOD_CAPS|SDL_KMOD_SHIFT|SDL_KMOD_ALT|SDL_KMOD_CTRL|SDL_KMOD_GUI)) == 0)
+						if((event.key.mod & (SDL_KMOD_CAPS|SDL_KMOD_SHIFT|SDL_KMOD_ALT|SDL_KMOD_CTRL|SDL_KMOD_GUI)) == 0)
 						{
 							// also, only do this if no modifiers like shift or AltGr or whatever are pressed
 							// so kc will most likely be the ascii char generated by this and can be ignored
@@ -718,8 +839,8 @@ IN_Update(void)
 				qboolean down = (event.type == SDL_EVENT_GAMEPAD_BUTTON_DOWN);
 				unsigned char btn = event.gbutton.button;
 
-				// Handle Back Button, to override its original key
-				Key_Event( (btn == sdl_back_button)? K_JOY_BACK : K_BTN_A + btn,
+				// Handle Esc button first, to override its original key
+				Key_Event( (btn == joy_escbutton)? K_ESCAPE : K_JOY_FIRST_BTN + btn,
 					down, true );
 				break;
 			}
@@ -731,26 +852,12 @@ IN_Update(void)
 				switch (event.gaxis.axis)
 				{
 					case SDL_GAMEPAD_AXIS_LEFT_TRIGGER :
-					{
-						qboolean new_left_trigger = axis_value > 8192;
-						if (new_left_trigger != left_trigger)
-						{
-							left_trigger = new_left_trigger;
-							Key_Event(K_TRIG_LEFT, left_trigger, true);
-						}
+						IN_VirtualKeyEvent(K_TRIG_LEFT, &left_trigger, axis_value > 8192);
 						break;
-					}
 
 					case SDL_GAMEPAD_AXIS_RIGHT_TRIGGER :
-					{
-						qboolean new_right_trigger = axis_value > 8192;
-						if (new_right_trigger != right_trigger)
-						{
-							right_trigger = new_right_trigger;
-							Key_Event(K_TRIG_RIGHT, right_trigger, true);
-						}
+						IN_VirtualKeyEvent(K_TRIG_RIGHT, &right_trigger, axis_value > 8192);
 						break;
-					}
 				}
 
 				if (!cl_paused->value && cls.key_dest == key_game)
@@ -770,11 +877,29 @@ IN_Update(void)
 							joystick_right_y = axis_value;
 							break;
 					}
+					break;
+				}
+
+				// Virtual keys to navigate menus with left stick
+				if (cls.key_dest == key_menu)
+				{
+					switch (event.gaxis.axis)
+					{
+						case SDL_GAMEPAD_AXIS_LEFTX :
+							IN_VirtualKeyEvent(K_LEFTARROW, &left_stick[0], axis_value < -16896);
+							IN_VirtualKeyEvent(K_RIGHTARROW, &left_stick[1], axis_value > 16896);
+							break;
+
+						case SDL_GAMEPAD_AXIS_LEFTY :
+							IN_VirtualKeyEvent(K_UPARROW, &left_stick[2], axis_value < -16896);
+							IN_VirtualKeyEvent(K_DOWNARROW, &left_stick[3], axis_value > 16896);
+							break;
+					}
 				}
 				break;
 			}
 
-#ifdef NATIVE_SDL_GYRO	// controller sensors' reading supported (gyro, accelerometer)
+#ifndef NO_SDL_GYRO	// gamepad sensors' reading is supported (gyro, accelerometer)
 			case SDL_EVENT_GAMEPAD_SENSOR_UPDATE :
 				if (event.gsensor.sensor != SDL_SENSOR_GYRO)
 				{
@@ -789,11 +914,11 @@ IN_Update(void)
 					break;
 				}
 
-#else	// gyro read as "secondary joystick"
+#else	// gyro read from a "secondary joystick" (usually with name ending in "IMU")
 			case SDL_EVENT_JOYSTICK_AXIS_MOTION :
-				if ( !imu_joystick || event.gdevice.which != SDL_GetJoystickInstanceID(imu_joystick) )
+				if ( !imu_joystick || event.gdevice.which != SDL_GetJoystickID(imu_joystick) )
 				{
-					break;	// controller axes handled by SDL_CONTROLLERAXISMOTION
+					break;	// gamepad axes handled by SDL_EVENT_GAMEPAD_AXIS_MOTION
 				}
 
 				int axis_value = event.gaxis.value;
@@ -816,12 +941,11 @@ IN_Update(void)
 					break;
 				}
 
-#endif	// NATIVE_SDL_GYRO
+#endif	// !NO_SDL_GYRO
 
-				if (gyro_active && gyro_mode->value &&
-					!cl_paused->value && cls.key_dest == key_game)
+				if (gyro_active && !cl_paused->value && cls.key_dest == key_game)
 				{
-#ifdef NATIVE_SDL_GYRO
+#ifndef NO_SDL_GYRO
 					if (!gyro_turning_axis->value)
 					{
 						gyro_yaw = event.gsensor.data[1] - gyro_calibration_y->value;		// yaw
@@ -832,7 +956,7 @@ IN_Update(void)
 					}
 					gyro_pitch = event.gsensor.data[0] - gyro_calibration_x->value;
 #else	// old "joystick" gyro
-					switch (event.gaxis.axis)	// inside "case SDL_JOYAXISMOTION" here
+					switch (event.gaxis.axis)	// inside "case SDL_EVENT_JOYSTICK_AXIS_MOTION" here
 					{
 						case IMU_JOY_AXIS_GYRO_PITCH:
 							gyro_pitch = -(axis_value - gyro_calibration_x->value);
@@ -849,7 +973,7 @@ IN_Update(void)
 								gyro_yaw = axis_value - gyro_calibration_z->value;
 							}
 					}
-#endif	// NATIVE_SDL_GYRO
+#endif	// !NO_SDL_GYRO
 				}
 				else
 				{
@@ -858,11 +982,7 @@ IN_Update(void)
 				break;
 
 			case SDL_EVENT_GAMEPAD_REMOVED :
-				if (!controller)
-				{
-					break;
-				}
-				if (event.gdevice.which == SDL_GetJoystickInstanceID(SDL_GetGamepadJoystick(controller))) {
+				if (controller && event.gdevice.which == SDL_GetJoystickID(SDL_GetGamepadJoystick(controller))) {
 					Cvar_SetValue("paused", 1);
 					IN_Controller_Shutdown(true);
 					IN_Controller_Init(false);
@@ -879,7 +999,7 @@ IN_Update(void)
 				break;
 
 			case SDL_EVENT_JOYSTICK_BATTERY_UPDATED :
-				if (!controller || event.jbattery.which != SDL_GetJoystickInstanceID(SDL_GetGamepadJoystick(controller)))
+				if (!controller || event.jbattery.which != SDL_GetJoystickID(SDL_GetGamepadJoystick(controller)))
 				{
 					break;
 				}
@@ -919,7 +1039,7 @@ IN_Update(void)
 	sys_frame_time = Sys_Milliseconds();
 
 	// Hot plugging delay handling, to not be "overwhelmed" because some controllers
-	// present themselves as two different devices, triggering SDL_JOYDEVICEADDED
+	// present themselves as two different devices, triggering SDL_EVENT_JOYSTICK_ADDED
 	// too many times. They could trigger it even at game initialization.
 	// Also used to keep time of the 'controller gyro calibration' pause.
 	if (updates_countdown)
@@ -943,7 +1063,7 @@ IN_Update(void)
 
 				case REASON_GYROCALIBRATION:	// finish and save calibration
 					{
-#ifdef NATIVE_SDL_GYRO
+#ifndef NO_SDL_GYRO
 						const float inverseSamples = 1.f / num_samples;
 						Cvar_SetValue("gyro_calibration_x", gyro_accum[0] * inverseSamples);
 						Cvar_SetValue("gyro_calibration_y", gyro_accum[1] * inverseSamples);
@@ -951,7 +1071,7 @@ IN_Update(void)
 #else
 						if (!num_samples[0] || !num_samples[1] || !num_samples[2])
 						{
-							Com_Printf("Calibration failed, please retry inside a level after having moved your controller a little.\n");
+							Com_Printf("Calibration failed, please retry inside a level after having moved your gamepad a little.\n");
 						}
 						else
 						{
@@ -972,6 +1092,20 @@ IN_Update(void)
 			countdown_reason = REASON_NONE;
 		}
 	}
+
+	// Gamepad labels' type and "confirm & cancel style" change handling
+	if (joy_labels->modified)
+	{
+		IN_GamepadLabels_Changed();
+	}
+	if (joy_confirm->modified)
+	{
+		IN_GamepadConfirm_Changed();
+	}
+	if (gyro_mode->modified)
+	{
+		IN_GyroMode_Changed();
+	}
 }
 
 /*
@@ -1068,7 +1202,11 @@ IN_TightenInput(float yaw, float pitch)
 {
 	thumbstick_t input = { yaw, pitch };
 	const float magnitude = IN_StickMagnitude(input);
+#ifndef NO_SDL_GYRO
 	const float threshold = (M_PI / 180.0f) * gyro_tightening->value;
+#else
+	const float threshold = (2560.0f / 180.0f) * gyro_tightening->value;
+#endif
 
 	if (magnitude < threshold)
 	{
@@ -1148,7 +1286,7 @@ IN_FlickStick(thumbstick_t stick, float axial_deadzone)
 	if (IN_StickMagnitude(stick) > Q_min(joy_flick_threshold->value, 1.0f))	// flick!
 	{
 		// Make snap-to-axis only if player wasn't already flicking
-		if (!is_flicking || flick_progress < FLICK_TIME)
+		if (!is_flicking || flick_progress < 1.0f)
 		{
 			processed = IN_SlopedAxialDeadzone(stick, axial_deadzone);
 		}
@@ -1159,7 +1297,8 @@ IN_FlickStick(thumbstick_t stick, float axial_deadzone)
 		{
 			// Flicking begins now, with a new target
 			is_flicking = true;
-			flick_progress = 0;
+			flick_progress = 0.0f;
+			started_flick = sys_frame_time;
 			target_angle = stick_angle;
 			IN_ResetSmoothSamples();
 		}
@@ -1197,12 +1336,6 @@ IN_Move(usercmd_t *cmd)
 	// Factor used to transform from SDL joystick input ([-32768, 32767])  to [-1, 1] range
 	static const float normalize_sdl_axis = 1.0f / 32768.0f;
 
-	// Flick Stick's factors to change to the target angle with a feeling of "ease out"
-	static const float rotation_factor[FLICK_TIME] =
-	{
-		0.305555556f, 0.249999999f, 0.194444445f, 0.138888889f, 0.083333333f, 0.027777778f
-	};
-
 	static float old_mouse_x;
 	static float old_mouse_y;
 	static float joystick_yaw, joystick_pitch;
@@ -1357,7 +1490,7 @@ IN_Move(usercmd_t *cmd)
 	//
 	// For movement this is not needed, as those are absolute values independent of framerate
 	float joyViewFactor = cls.rframetime/0.01666f;
-#ifdef NATIVE_SDL_GYRO
+#ifndef NO_SDL_GYRO
 	float gyroViewFactor = (1.0f / M_PI) * joyViewFactor;
 #else
 	float gyroViewFactor = (1.0f / 2560.0f) * joyViewFactor;	// normalized for Switch gyro
@@ -1407,10 +1540,25 @@ IN_Move(usercmd_t *cmd)
 	}
 
 	// Flick Stick: flick in progress, changing the yaw angle to the target progressively
-	if (flick_progress < FLICK_TIME)
+	if (flick_progress < 1.0f)
 	{
-		cl.viewangles[YAW] += target_angle * rotation_factor[flick_progress];
-		flick_progress++;
+		float cur_progress = (float)(sys_frame_time - started_flick) / FLICK_TIME;
+
+		if (cur_progress > 1.0f)
+		{
+			cur_progress = 1.0f;
+		}
+		else
+		{
+			// "Ease out" warp processing: f(x)=1-(1-x)^2 , 0 <= x <= 1
+			// http://gyrowiki.jibbsmart.com/blog:good-gyro-controls-part-2:the-flick-stick#toc0
+			cur_progress = 1.0f - cur_progress;
+			cur_progress *= cur_progress;
+			cur_progress = 1.0f - cur_progress;
+		}
+
+		cl.viewangles[YAW] += (cur_progress - flick_progress) * target_angle;
+		flick_progress = cur_progress;
 	}
 }
 
@@ -1994,7 +2142,7 @@ Controller_Rumble(const char *name, vec3_t source, qboolean from_player,
 	// Com_Printf("%-29s: vol %5u - %4u ms - dp %.3f l %5.0f h %5.0f\n",
 	//	name, effect_volume, duration, dist_prop, low_freq, hi_freq);
 
-	if (SDL_RumbleGamepad(controller, low_freq, hi_freq, duration) == -1)
+	if (!SDL_RumbleGamepad(controller, low_freq, hi_freq, duration))
 	{
 		if (!joystick_haptic)
 		{
@@ -2013,7 +2161,7 @@ Controller_Rumble(const char *name, vec3_t source, qboolean from_player,
 void
 StartCalibration(void)
 {
-#ifdef NATIVE_SDL_GYRO
+#ifndef NO_SDL_GYRO
 	num_samples = 0;
 #else
 	num_samples[0] = num_samples[1] = num_samples[2] = 0;
@@ -2041,21 +2189,21 @@ IN_Controller_Init(qboolean notify_user)
 	int nummappings;
 	char controllerdb[MAX_OSPATH] = {0};
 	SDL_Joystick *joystick = NULL;
-	SDL_bool is_controller = SDL_FALSE;
+	bool is_controller = false;
 
-	cvar = Cvar_Get("in_sdlbackbutton", "0", CVAR_ARCHIVE);
+	cvar = Cvar_Get("joy_escbutton", "0", CVAR_ARCHIVE);
 	if (cvar)
 	{
 		switch ((int)cvar->value)
 		{
 			case 1:
-				sdl_back_button = SDL_GAMEPAD_BUTTON_START;
+				joy_escbutton = SDL_GAMEPAD_BUTTON_BACK;
 				break;
 			case 2:
-				sdl_back_button = SDL_GAMEPAD_BUTTON_GUIDE;
+				joy_escbutton = SDL_GAMEPAD_BUTTON_GUIDE;
 				break;
 			default:
-				sdl_back_button = SDL_GAMEPAD_BUTTON_BACK;
+				joy_escbutton = SDL_GAMEPAD_BUTTON_START;
 		}
 	}
 
@@ -2067,22 +2215,14 @@ IN_Controller_Init(qboolean notify_user)
 
 	if (notify_user)
 	{
-		Com_Printf("- Game Controller init attempt -\n");
+		Com_Printf("- Gamepad init attempt -\n");
 	}
 
 	if (!SDL_WasInit(SDL_INIT_GAMEPAD | SDL_INIT_HAPTIC))
 	{
-
-#ifdef SDL_HINT_JOYSTICK_HIDAPI_PS4_RUMBLE	// extended input reports on PS controllers (enables gyro thru bluetooth)
-		SDL_SetHint( SDL_HINT_JOYSTICK_HIDAPI_PS4_RUMBLE, "1" );
-#endif
-#ifdef SDL_HINT_JOYSTICK_HIDAPI_PS5_RUMBLE
-		SDL_SetHint( SDL_HINT_JOYSTICK_HIDAPI_PS5_RUMBLE, "1" );
-#endif
-
-		if (SDL_Init(SDL_INIT_GAMEPAD | SDL_INIT_HAPTIC) == -1)
+		if (!SDL_Init(SDL_INIT_GAMEPAD | SDL_INIT_HAPTIC))
 		{
-			Com_Printf ("Couldn't init SDL Game Controller: %s.\n", SDL_GetError());
+			Com_Printf ("Couldn't init SDL Gamepad: %s.\n", SDL_GetError());
 			return;
 		}
 	}
@@ -2114,6 +2254,8 @@ IN_Controller_Init(qboolean notify_user)
 			show_haptic = true;
 		}
 
+		SDL_free((void *)joysticks);
+
 		return;
 	}
 
@@ -2140,27 +2282,36 @@ IN_Controller_Init(qboolean notify_user)
 		Com_Printf ("Trying joystick %d, '%s'\n", i+1, joystick_name);
 
 		// Ugly hack to detect IMU-only devices - works for Switch controllers at least
-		if (name_len > 4 && !strncmp(joystick_name + name_len - 4, " IMU", 4))
+		if ( name_len > 6 && strstr(joystick_name + name_len - 6, "IMU") )
 		{
+#ifndef NO_SDL_GYRO
 			SDL_CloseJoystick(joystick);
 			joystick = NULL;
-#ifdef NATIVE_SDL_GYRO
 			Com_Printf ("Skipping IMU device.\n");
+
 #else	// if it's not a Left JoyCon, use it as Gyro
-			Com_Printf ("IMU device found.\n");
-			if ( !imu_joystick && name_len > 16 && strncmp(joystick_name + name_len - 16, "Left Joy-Con IMU", 16) != 0 )
+			qboolean using_imu = !imu_joystick && !( strstr(joystick_name, "Joy-Con") && strstr(joystick_name, "L") );
+			Com_Printf ("IMU device found... ");
+			SDL_CloseJoystick(joystick);
+			joystick = NULL;
+
+			if (using_imu)
 			{
 				imu_joystick = SDL_OpenJoystick(joysticks[i]);
 				if (imu_joystick)
 				{
 					show_gyro = true;
-					Com_Printf ("Using this device as Gyro sensor.\n");
+					Com_Printf ("using it as Gyro sensor.\n");
 				}
 				else
 				{
-					Com_Printf ("Couldn't open IMU: %s.\n", SDL_GetError());
+					Com_Printf ("\nCouldn't open IMU: %s.\n", SDL_GetError());
 				}
 			}
+			else
+			{
+				Com_Printf ("skipping.\n");
+			}
 #endif
 			continue;
 		}
@@ -2172,11 +2323,11 @@ IN_Controller_Init(qboolean notify_user)
 		if (!is_controller)
 		{
 			char joystick_guid[65] = {0};
-			SDL_JoystickGUID guid = SDL_GetJoystickInstanceGUID(joysticks[i]);
+			SDL_GUID guid = SDL_GetJoystickGUIDForID(joysticks[i]);
 
-			SDL_GetJoystickGUIDString(guid, joystick_guid, 64);
+			SDL_GUIDToString(guid, joystick_guid, 64);
 
-			Com_Printf ("To use joystick as game controller, provide its config by either:\n"
+			Com_Printf ("To identify joystick as Gamepad, provide its config by either:\n"
 				" * Putting 'gamecontrollerdb.txt' file in your game directory.\n"
 				" * Or setting SDL_GAMECONTROLLERCONFIG environment variable. E.g.:\n");
 			Com_Printf ("SDL_GAMECONTROLLERCONFIG='%s,%s,leftx:a0,lefty:a1,rightx:a2,righty:a3,back:b1,...'\n", joystick_guid, joystick_name);
@@ -2190,18 +2341,18 @@ IN_Controller_Init(qboolean notify_user)
 			controller = SDL_OpenGamepad(joysticks[i]);
 			if (!controller)
 			{
-				Com_Printf("SDL Controller error: %s.\n", SDL_GetError());
+				Com_Printf("SDL Gamepad error: %s.\n", SDL_GetError());
 				continue;	// try next joystick
 			}
 
 			show_gamepad = true;
-			Com_Printf("Enabled as Game Controller, settings:\n%s\n",
+			Com_Printf("Enabled as Gamepad, settings:\n%s\n",
 				   SDL_GetGamepadMapping(controller));
 
-#ifdef NATIVE_SDL_GYRO
+#ifndef NO_SDL_GYRO
 
 			if (SDL_GamepadHasSensor(controller, SDL_SENSOR_GYRO)
-				&& !SDL_SetGamepadSensorEnabled(controller, SDL_SENSOR_GYRO, SDL_TRUE) )
+				&& SDL_SetGamepadSensorEnabled(controller, SDL_SENSOR_GYRO, true) )
 			{
 				show_gyro = true;
 				Com_Printf( "Gyro sensor enabled at %.2f Hz\n",
@@ -2212,13 +2363,13 @@ IN_Controller_Init(qboolean notify_user)
 				Com_Printf("Gyro sensor not found.\n");
 			}
 
-			SDL_bool hasLED = SDL_GetBooleanProperty(SDL_GetGamepadProperties(controller), SDL_PROP_JOYSTICK_CAP_RGB_LED_BOOLEAN, SDL_FALSE);
+			bool hasLED = SDL_GetBooleanProperty(SDL_GetGamepadProperties(controller), SDL_PROP_JOYSTICK_CAP_RGB_LED_BOOLEAN, false);
 			if (hasLED)
 			{
 				SDL_SetGamepadLED(controller, 0, 80, 0);	// green light
 			}
 
-#endif	// NATIVE_SDL_GYRO
+#endif	// !NO_SDL_GYRO
 
 			joystick_haptic = SDL_OpenHapticFromJoystick(SDL_GetGamepadJoystick(controller));
 
@@ -2236,7 +2387,7 @@ IN_Controller_Init(qboolean notify_user)
 				show_haptic = true;
 			}
 
-			SDL_bool hasRumble = SDL_GetBooleanProperty(SDL_GetGamepadProperties(controller), SDL_PROP_GAMEPAD_CAP_RUMBLE_BOOLEAN, SDL_FALSE);
+			bool hasRumble = SDL_GetBooleanProperty(SDL_GetGamepadProperties(controller), SDL_PROP_GAMEPAD_CAP_RUMBLE_BOOLEAN, false);
 			if (hasRumble)
 			{
 				show_haptic = true;
@@ -2244,16 +2395,19 @@ IN_Controller_Init(qboolean notify_user)
 			}
 			else
 			{
-				Com_Printf("Controller doesn't support rumble.\n");
+				Com_Printf("Gamepad doesn't support rumble.\n");
 			}
 
-#ifdef NATIVE_SDL_GYRO	// "native" exits when finding a single working controller
+#ifndef NO_SDL_GYRO	// "native SDL gyro" exits when finding a single working gamepad
 			break;
 #endif
 		}
 	}
 
 	SDL_free((void *)joysticks);
+	IN_GamepadLabels_Changed();
+	IN_GamepadConfirm_Changed();
+	IN_GyroMode_Changed();
 }
 
 /*
@@ -2284,11 +2438,13 @@ IN_Init(void)
 	joy_haptic_distance = Cvar_Get("joy_haptic_distance", "100.0", CVAR_ARCHIVE);
 	haptic_feedback_filter = Cvar_Get("joy_haptic_filter", default_haptic_filter, CVAR_ARCHIVE);
 
-	joy_yawsensitivity = Cvar_Get("joy_yawsensitivity", "1.0", CVAR_ARCHIVE);
-	joy_pitchsensitivity = Cvar_Get("joy_pitchsensitivity", "1.0", CVAR_ARCHIVE);
+	joy_yawsensitivity = Cvar_Get("joy_yawsensitivity", "2.5", CVAR_ARCHIVE);
+	joy_pitchsensitivity = Cvar_Get("joy_pitchsensitivity", "2.5", CVAR_ARCHIVE);
 	joy_forwardsensitivity = Cvar_Get("joy_forwardsensitivity", "1.0", CVAR_ARCHIVE);
 	joy_sidesensitivity = Cvar_Get("joy_sidesensitivity", "1.0", CVAR_ARCHIVE);
 
+	joy_labels = Cvar_Get("joy_labels", "-1", CVAR_ARCHIVE);
+	joy_confirm = Cvar_Get("joy_confirm", "-1", CVAR_ARCHIVE);
 	joy_layout = Cvar_Get("joy_layout", "0", CVAR_ARCHIVE);
 	joy_left_expo = Cvar_Get("joy_left_expo", "2.0", CVAR_ARCHIVE);
 	joy_left_snapaxis = Cvar_Get("joy_left_snapaxis", "0.15", CVAR_ARCHIVE);
@@ -2303,16 +2459,11 @@ IN_Init(void)
 	gyro_calibration_y = Cvar_Get("gyro_calibration_y", "0.0", CVAR_ARCHIVE);
 	gyro_calibration_z = Cvar_Get("gyro_calibration_z", "0.0", CVAR_ARCHIVE);
 
-	gyro_yawsensitivity = Cvar_Get("gyro_yawsensitivity", "1.0", CVAR_ARCHIVE);
-	gyro_pitchsensitivity = Cvar_Get("gyro_pitchsensitivity", "1.0", CVAR_ARCHIVE);
+	gyro_yawsensitivity = Cvar_Get("gyro_yawsensitivity", "2.5", CVAR_ARCHIVE);
+	gyro_pitchsensitivity = Cvar_Get("gyro_pitchsensitivity", "2.5", CVAR_ARCHIVE);
 	gyro_tightening = Cvar_Get("gyro_tightening", "3.5", CVAR_ARCHIVE);
 	gyro_turning_axis = Cvar_Get("gyro_turning_axis", "0", CVAR_ARCHIVE);
-
 	gyro_mode = Cvar_Get("gyro_mode", "2", CVAR_ARCHIVE);
-	if ((int)gyro_mode->value == 2)
-	{
-		gyro_active = true;
-	}
 
 	windowed_pauseonfocuslost = Cvar_Get("vid_pauseonfocuslost", "0", CVAR_USERINFO | CVAR_ARCHIVE);
 	windowed_mouse = Cvar_Get("windowed_mouse", "1", CVAR_USERINFO | CVAR_ARCHIVE);
@@ -2333,8 +2484,6 @@ IN_Init(void)
 		}
 	}
 
-	SDL_StartTextInput();
-
 	IN_Controller_Init(false);
 
 	Com_Printf("------------------------------------\n\n");
@@ -2360,7 +2509,7 @@ IN_Controller_Shutdown(qboolean notify_user)
 {
 	if (notify_user)
 	{
-		Com_Printf("- Game Controller disconnected -\n");
+		Com_Printf("- Gamepad disconnected -\n");
 	}
 
 	IN_Haptic_Shutdown();
@@ -2374,7 +2523,7 @@ IN_Controller_Shutdown(qboolean notify_user)
 	joystick_left_x = joystick_left_y = joystick_right_x = joystick_right_y = 0;
 	gyro_yaw = gyro_pitch = 0;
 
-#ifndef NATIVE_SDL_GYRO
+#ifdef NO_SDL_GYRO
 	if (imu_joystick)
 	{
 		SDL_CloseJoystick(imu_joystick);
@@ -2410,3 +2559,30 @@ IN_Shutdown(void)
 }
 
 /* ------------------------------------------------------------------ */
+
+void
+IN_GetClipboardText(char *out, size_t n)
+{
+	char *s = SDL_GetClipboardText();
+
+	if (!s || *s == '\0')
+	{
+		*out = '\0';
+		return;
+	}
+
+	Q_strlcpy(out, s, n - 1);
+
+	SDL_free(s);
+}
+
+/* Copy string s to the clipboard.
+   Returns 0 on success, 1 otherwise.
+*/
+int
+IN_SetClipboardText(const char *s)
+{
+	bool res = SDL_SetClipboardText(s);
+
+	return !res;
+}
