@@ -47,9 +47,17 @@ qboolean R_Upload32(unsigned *data, int width, int height, qboolean mipmap);
 int gl_solid_format = GL_RGB;
 int gl_alpha_format = GL_RGBA;
 
-int gl_tex_solid_format = GL_RGB;
+#ifdef YQ2_GL1_GLES
+#define DEFAULT_SOLID_FORMAT GL_RGBA
+#else
+#define DEFAULT_SOLID_FORMAT GL_RGB
+#endif
+
+int gl_tex_solid_format = DEFAULT_SOLID_FORMAT;
 int gl_tex_alpha_format = GL_RGBA;
 
+#undef DEFAULT_SOLID_FORMAT
+
 int gl_filter_min = GL_LINEAR_MIPMAP_NEAREST;
 int gl_filter_max = GL_LINEAR;
 
@@ -76,6 +84,20 @@ typedef struct
 	int mode;
 } gltmode_t;
 
+#ifdef YQ2_GL1_GLES
+
+gltmode_t gl_alpha_modes[] = {
+	{"default", GL_RGBA},
+	{"GL_RGBA", GL_RGBA},
+};
+
+gltmode_t gl_solid_modes[] = {
+	{"default", GL_RGBA},
+	{"GL_RGBA", GL_RGBA},
+};
+
+#else
+
 gltmode_t gl_alpha_modes[] = {
 	{"default", GL_RGBA},
 	{"GL_RGBA", GL_RGBA},
@@ -85,8 +107,6 @@ gltmode_t gl_alpha_modes[] = {
 	{"GL_RGBA2", GL_RGBA2},
 };
 
-#define NUM_GL_ALPHA_MODES (sizeof(gl_alpha_modes) / sizeof(gltmode_t))
-
 gltmode_t gl_solid_modes[] = {
 	{"default", GL_RGB},
 	{"GL_RGB", GL_RGB},
@@ -96,6 +116,9 @@ gltmode_t gl_solid_modes[] = {
 	{"GL_R3_G3_B2", GL_R3_G3_B2},
 };
 
+#endif
+
+#define NUM_GL_ALPHA_MODES (sizeof(gl_alpha_modes) / sizeof(gltmode_t))
 #define NUM_GL_SOLID_MODES (sizeof(gl_solid_modes) / sizeof(gltmode_t))
 
 typedef struct
@@ -447,7 +470,7 @@ R_ImageList_f(void)
 /*
  * Fill background pixels so mipmapping doesn't have haloes
  */
-void
+static void
 R_FloodFillSkin(byte *skin, int skinwidth, int skinheight)
 {
 	byte fillcolor = *skin; /* assume this is the pixel to fill */
@@ -513,7 +536,7 @@ R_FloodFillSkin(byte *skin, int skinwidth, int skinheight)
  * texture to increase the
  * lighting range
  */
-void
+static void
 R_LightScaleTexture(unsigned *in, int inwidth,
 		int inheight, qboolean only_gamma)
 {
@@ -554,7 +577,7 @@ R_LightScaleTexture(unsigned *in, int inwidth,
 /*
  * Operates in place, quartering the size of the texture
  */
-void
+static void
 R_MipMap(byte *in, int width, int height)
 {
 	int i, j;
@@ -579,7 +602,7 @@ R_MipMap(byte *in, int width, int height)
 /*
  * Returns has_alpha
  */
-void
+static void
 R_BuildPalettedTexture(unsigned char *paletted_texture, unsigned char *scaled,
 		int scaled_width, int scaled_height)
 {
@@ -601,7 +624,7 @@ R_BuildPalettedTexture(unsigned char *paletted_texture, unsigned char *scaled,
 	}
 }
 
-qboolean
+static qboolean
 R_Upload32Native(unsigned *data, int width, int height, qboolean mipmap)
 {
 	// This is for GL 2.x so no palettes, no scaling, no messing around with the data here. :)
@@ -637,7 +660,7 @@ R_Upload32Native(unsigned *data, int width, int height, qboolean mipmap)
 }
 
 
-qboolean
+static qboolean
 R_Upload32Soft(unsigned *data, int width, int height, qboolean mipmap)
 {
 	int samples;
@@ -1103,7 +1126,8 @@ image_t *
 R_FindImage(const char *name, imagetype_t type)
 {
 	image_t *image;
-	int i, len;
+	size_t len;
+	int i;
 	char *ptr;
 	char namewe[256];
 	const char* ext;
@@ -1162,7 +1186,7 @@ R_FindImage(const char *name, imagetype_t type)
 }
 
 struct image_s *
-RI_RegisterSkin(char *name)
+RI_RegisterSkin(const char *name)
 {
 	return R_FindImage(name, it_skin);
 }
