@@ -28,12 +28,18 @@
 
 #include "header/local.h"
 
+typedef struct
+{
+	int top, bottom, left, right;
+} lmrect_t;
+
 int c_visible_lightmaps;
 int c_visible_textures;
 static vec3_t modelorg; /* relative to viewpoint */
 msurface_t *r_alpha_surfaces;
 
 gllightmapstate_t gl_lms;
+extern int cur_lm_copy;
 
 void LM_InitBlock(void);
 void LM_UploadBlock(qboolean dynamic);
@@ -69,8 +75,8 @@ R_DrawGLPoly(msurface_t *fa)
 
 	for ( i = 0; i < nv; i++, v += VERTEXSIZE )
 	{
-		R_BufferVertex(v[0], v[1], v[2]);
-		R_BufferSingleTex(v[3] + scroll, v[4]);
+		GLBUFFER_VERTEX( v[0], v[1], v[2] )
+		GLBUFFER_SINGLETEX( v[3] + scroll, v[4] )
 	}
 }
 
@@ -108,7 +114,7 @@ R_DrawTriangleOutlines(void)
 
                     for (k=0; k<3; k++)
                     {
-                       vtx[0+k] = p->verts [ 0 ][ k ];
+                        vtx[0+k] = p->verts [ 0 ][ k ];
                         vtx[3+k] = p->verts [ j - 1 ][ k ];
                         vtx[6+k] = p->verts [ j ][ k ];
                         vtx[9+k] = p->verts [ 0 ][ k ];
@@ -382,7 +388,7 @@ R_BlendLightmaps(const model_t *currentmodel)
 }
 
 static void
-R_RenderBrushPoly(entity_t *currententity, msurface_t *fa)
+R_RenderBrushPoly(msurface_t *fa)
 {
 	int maps;
 	qboolean is_dynamic = false;
@@ -519,57 +525,11 @@ R_DrawAlphaSurfaces(void)
 	r_alpha_surfaces = NULL;
 }
 
-static qboolean
-R_HasDynamicLights(msurface_t *surf, int *mapNum)
-{
-	int map;
-	qboolean is_dynamic = false;
-
-	if ( r_fullbright->value || !gl1_dynamic->value ||
-		(surf->texinfo->flags & (SURF_SKY | SURF_TRANS33 | SURF_TRANS66 | SURF_WARP)) )
-	{
-		return false;
-	}
-
-	// Any dynamic lights on this surface?
-	for (map = 0; map < MAXLIGHTMAPS && surf->styles[map] != 255; map++)
-	{
-		if (r_newrefdef.lightstyles[surf->styles[map]].white != surf->cached_light[map])
-		{
-			is_dynamic = true;
-			break;
-		}
-	}
-
-	// No matter if it is this frame or was in the previous one: has dynamic lights
-	if ( !is_dynamic && (surf->dlightframe == r_framecount || surf->dirty_lightmap) )
-	{
-		is_dynamic = true;
-	}
-
-	if (mapNum)
-	{
-		*mapNum = map;
-	}
-	return is_dynamic;
-}
-
 static void
-R_UpdateSurfCache(msurface_t *surf, int map)
-{
-	if ( ((surf->styles[map] >= 32) || (surf->styles[map] == 0))
-		&& (surf->dlightframe != r_framecount) )
-	{
-		R_SetCacheState(surf);
-	}
-	surf->dirty_lightmap = (surf->dlightframe == r_framecount);
-}
-
-static void
-R_RenderLightmappedPoly(entity_t *currententity, msurface_t *surf)
+R_RenderLightmappedPoly(msurface_t *surf)
 {
+	const int nv = surf->polys->numverts;
 	int i;
-	int nv = surf->polys->numverts;
 	float scroll;
 	float *v;
 
@@ -594,8 +554,30 @@ R_RenderLightmappedPoly(entity_t *currententity, msurface_t *surf)
 
 	for (i = 0; i < nv; i++, v += VERTEXSIZE)
 	{
-		R_BufferVertex( v[0], v[1], v[2] );
-		R_BufferMultiTex( v[3] + scroll, v[4], v[5], v[6] );
+		GLBUFFER_VERTEX( v[0], v[1], v[2] )
+		GLBUFFER_MULTITEX( v[3] + scroll, v[4], v[5], v[6] )
+	}
+}
+
+/* Add "adding" area to "obj" */
+static void
+R_JoinAreas(lmrect_t *adding, lmrect_t *obj)
+{
+	if (adding->top < obj->top)
+	{
+		obj->top = adding->top;
+	}
+	if (adding->bottom > obj->bottom)
+	{
+		obj->bottom = adding->bottom;
+	}
+	if (adding->left < obj->left)
+	{
+		obj->left = adding->left;
+	}
+	if (adding->right > obj->right)
+	{
+		obj->right = adding->right;
 	}
 }
 
@@ -603,16 +585,33 @@ R_RenderLightmappedPoly(entity_t *currententity, msurface_t *surf)
 static void
 R_RegenAllLightmaps()
 {
-	int i, map, smax, tmax, top, bottom, left, right, bt, bb, bl, br;
-	qboolean affected_lightmap, pixelstore_set = false;
+	static lmrect_t lmchange[MAX_LIGHTMAPS][MAX_LIGHTMAP_COPIES];
+	static qboolean altered[MAX_LIGHTMAPS][MAX_LIGHTMAP_COPIES];
+
+	int i, map, smax, tmax, lmtex;
+	lmrect_t current, best;
 	msurface_t *surf;
 	byte *base;
+	qboolean affected_lightmap;
+#ifndef YQ2_GL1_GLES
+	qboolean pixelstore_set = false;
+#endif
 
-	if ( !gl_config.multitexture )
+	if ( !gl_config.multitexture || r_fullbright->value || !gl1_dynamic->value )
 	{
 		return;
 	}
 
+	if (gl_config.lightmapcopies)
+	{
+		cur_lm_copy = (cur_lm_copy + 1) % MAX_LIGHTMAP_COPIES;	// select the next lightmap copy
+		lmtex = gl_state.max_lightmaps * cur_lm_copy;	// ...and its corresponding texture
+	}
+	else
+	{
+		lmtex = 0;
+	}
+
 	for (i = 1; i < gl_state.max_lightmaps; i++)
 	{
 		if (!gl_lms.lightmap_surfaces[i] || !gl_lms.lightmap_buffer[i])
@@ -621,80 +620,134 @@ R_RegenAllLightmaps()
 		}
 
 		affected_lightmap = false;
-		bt = gl_state.block_height;
-		bl = gl_state.block_width;
-		bb = br = 0;
+		best.top = gl_state.block_height;
+		best.left = gl_state.block_width;
+		best.bottom = best.right = 0;
 
 		for (surf = gl_lms.lightmap_surfaces[i];
 			 surf != 0;
 			 surf = surf->lightmapchain)
 		{
-			if ( !R_HasDynamicLights(surf, &map) )
+			if (surf->texinfo->flags & (SURF_SKY | SURF_TRANS33 | SURF_TRANS66 | SURF_WARP))
 			{
 				continue;
 			}
 
+			// Any dynamic lights on this surface?
+			for (map = 0; map < MAXLIGHTMAPS && surf->styles[map] != 255; map++)
+			{
+				if (r_newrefdef.lightstyles[surf->styles[map]].white != surf->cached_light[map])
+				{
+					goto dynamic_surf;
+				}
+			}
+
+			// Doesn't matter if it is this frame or was in the previous one: surface has dynamic lights
+			if ( surf->dlightframe == r_framecount || surf->dirty_lightmap )
+			{
+				goto dynamic_surf;
+			}
+
+			continue;	// no dynamic lights affect this surface in this frame
+
+dynamic_surf:
 			affected_lightmap = true;
 			smax = (surf->extents[0] >> 4) + 1;
 			tmax = (surf->extents[1] >> 4) + 1;
 
-			left = surf->light_s;
-			right = surf->light_s + smax;
-			top = surf->light_t;
-			bottom = surf->light_t + tmax;
+			current.left = surf->light_s;
+			current.right = surf->light_s + smax;
+			current.top = surf->light_t;
+			current.bottom = surf->light_t + tmax;
 
 			base = gl_lms.lightmap_buffer[i];
-			base += (top * gl_state.block_width + left) * LIGHTMAP_BYTES;
+			base += (current.top * gl_state.block_width + current.left) * LIGHTMAP_BYTES;
 
 			R_BuildLightMap(surf, base, gl_state.block_width * LIGHTMAP_BYTES);
-			R_UpdateSurfCache(surf, map);
 
-			if (left < bl)
+			if ( ((surf->styles[map] >= 32) || (surf->styles[map] == 0))
+				&& (surf->dlightframe != r_framecount) )
 			{
-				bl = left;
-			}
-			if (right > br)
-			{
-				br = right;
-			}
-			if (top < bt)
-			{
-				bt = top;
-			}
-			if (bottom > bb)
-			{
-				bb = bottom;
+				R_SetCacheState(surf);
 			}
+
+			surf->dirty_lightmap = (surf->dlightframe == r_framecount);
+			R_JoinAreas(&current, &best);
 		}
 
-		if (!affected_lightmap)
+		if (!gl_config.lightmapcopies && !affected_lightmap)
 		{
 			continue;
 		}
 
+		if (gl_config.lightmapcopies)
+		{
+			// Add all the changes that have happened in the last few frames,
+			// at least just for consistency between them.
+			qboolean apply_changes = affected_lightmap;
+			current = best;		// save state for next frames... +
+
+			for (int k = 0; k < MAX_LIGHTMAP_COPIES; k++)
+			{
+				if (altered[i][k])
+				{
+					apply_changes = true;
+					R_JoinAreas(&lmchange[i][k], &best);
+				}
+			}
+
+			altered[i][cur_lm_copy] = affected_lightmap;
+			if (affected_lightmap)
+			{
+				lmchange[i][cur_lm_copy] = current;	// + ...here
+			}
+
+			if (!apply_changes)
+			{
+				continue;
+			}
+		}
+
+#ifndef YQ2_GL1_GLES
 		if (!pixelstore_set)
 		{
 			glPixelStorei(GL_UNPACK_ROW_LENGTH, gl_state.block_width);
 			pixelstore_set = true;
 		}
-
-		base = gl_lms.lightmap_buffer[i];
-		base += (bt * gl_state.block_width + bl) * LIGHTMAP_BYTES;
+#endif
 
 		// upload changes
-		R_Bind(gl_state.lightmap_textures + i);
-		glTexSubImage2D(GL_TEXTURE_2D, 0, bl, bt, br - bl, bb - bt,
-						GL_LIGHTMAP_FORMAT, GL_UNSIGNED_BYTE, base);
+		base = gl_lms.lightmap_buffer[i];
+
+#ifdef YQ2_GL1_GLES
+		base += (best.top * gl_state.block_width) * LIGHTMAP_BYTES;
+
+		R_Bind(gl_state.lightmap_textures + i + lmtex);
+
+		glTexSubImage2D(GL_TEXTURE_2D, 0, 0, best.top,
+			gl_state.block_width, best.bottom - best.top,
+			GL_LIGHTMAP_FORMAT, GL_UNSIGNED_BYTE, base);
+#else
+		base += (best.top * gl_state.block_width + best.left) * LIGHTMAP_BYTES;
+
+		R_Bind(gl_state.lightmap_textures + i + lmtex);
+
+		glTexSubImage2D(GL_TEXTURE_2D, 0, best.left, best.top,
+			best.right - best.left, best.bottom - best.top,
+			GL_LIGHTMAP_FORMAT, GL_UNSIGNED_BYTE, base);
+#endif
 	}
 
+#ifndef YQ2_GL1_GLES
 	if (pixelstore_set)
 	{
 		glPixelStorei(GL_UNPACK_ROW_LENGTH, 0);
 	}
+#endif
 }
 
 static void
-R_DrawTextureChains(entity_t *currententity)
+R_DrawTextureChains(void)
 {
 	int i;
 	msurface_t *s;
@@ -723,7 +776,7 @@ R_DrawTextureChains(entity_t *currententity)
 			for ( ; s; s = s->texturechain)
 			{
 				R_UpdateGLBuffer(buf_singletex, image->texnum, 0, s->flags, 1);
-				R_RenderBrushPoly(currententity, s);
+				R_RenderBrushPoly(s);
 			}
 
 			image->texturechain = NULL;
@@ -746,7 +799,7 @@ R_DrawTextureChains(entity_t *currententity)
 				if (!(s->flags & SURF_DRAWTURB))
 				{
 					R_UpdateGLBuffer(buf_mtex, image->texnum, s->lightmaptexturenum, 0, 1);
-					R_RenderLightmappedPoly(currententity, s);
+					R_RenderLightmappedPoly(s);
 				}
 			}
 		}
@@ -766,7 +819,7 @@ R_DrawTextureChains(entity_t *currententity)
 				if (s->flags & SURF_DRAWTURB)
 				{
 					R_UpdateGLBuffer(buf_singletex, image->texnum, 0, s->flags, 1);
-					R_RenderBrushPoly(currententity, s);
+					R_RenderBrushPoly(s);
 				}
 			}
 
@@ -834,12 +887,12 @@ R_DrawInlineBModel(entity_t *currententity, const model_t *currentmodel)
 				{
 					// Dynamic lighting already generated in R_GetBrushesLighting()
 					R_UpdateGLBuffer(buf_mtex, image->texnum, psurf->lightmaptexturenum, 0, 1);
-					R_RenderLightmappedPoly(currententity, psurf);
+					R_RenderLightmappedPoly(psurf);
 				}
 				else
 				{
 					R_UpdateGLBuffer(buf_singletex, image->texnum, 0, psurf->flags, 1);
-					R_RenderBrushPoly(currententity, psurf);
+					R_RenderBrushPoly(psurf);
 				}
 			}
 		}
@@ -1193,7 +1246,7 @@ R_DrawWorld(void)
 	R_RecursiveWorldNode(&ent, r_worldmodel->nodes);
 	R_GetBrushesLighting();
 	R_RegenAllLightmaps();
-	R_DrawTextureChains(&ent);
+	R_DrawTextureChains();
 	R_BlendLightmaps(r_worldmodel);
 	R_DrawSkyBox();
 	R_DrawTriangleOutlines();
