@@ -116,7 +116,7 @@ GL3_TextureMode(char *string)
 			/* Set anisotropic filter if supported and enabled */
 			if (gl3config.anisotropic && gl_anisotropic->value)
 			{
-				glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAX_ANISOTROPY_EXT, max(gl_anisotropic->value, 1.f));
+				glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAX_ANISOTROPY_EXT, Q_max(gl_anisotropic->value, 1.f));
 			}
 		}
 		else /* texture has no mipmaps */
@@ -186,7 +186,7 @@ GL3_BindLightmap(int lightmapnum)
 /*
  * Returns has_alpha
  */
-qboolean
+static qboolean
 GL3_Upload32(unsigned *data, int width, int height, qboolean mipmap)
 {
 	qboolean res;
@@ -227,7 +227,7 @@ GL3_Upload32(unsigned *data, int width, int height, qboolean mipmap)
 
 	if (mipmap && gl3config.anisotropic && gl_anisotropic->value)
 	{
-		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAX_ANISOTROPY_EXT, max(gl_anisotropic->value, 1.f));
+		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAX_ANISOTROPY_EXT, Q_max(gl_anisotropic->value, 1.f));
 	}
 
 	return res;
@@ -237,7 +237,7 @@ GL3_Upload32(unsigned *data, int width, int height, qboolean mipmap)
 /*
  * Returns has_alpha
  */
-qboolean
+static qboolean
 GL3_Upload8(byte *data, int width, int height, qboolean mipmap, qboolean is_sky)
 {
 	int s = width * height;
@@ -605,7 +605,8 @@ gl3image_t *
 GL3_FindImage(const char *name, imagetype_t type)
 {
 	gl3image_t *image;
-	int i, len;
+	size_t len;
+	int i;
 	char *ptr;
 	char namewe[256];
 	const char* ext;
@@ -664,7 +665,7 @@ GL3_FindImage(const char *name, imagetype_t type)
 }
 
 gl3image_t *
-GL3_RegisterSkin(char *name)
+GL3_RegisterSkin(const char *name)
 {
 	return GL3_FindImage(name, it_skin);
 }
