@@ -577,14 +577,14 @@ void	RE_BeginRegistration (char *model);
 struct model_s	*RE_RegisterModel (char *name);
 void	RE_EndRegistration (void);
 
-struct image_s	*RE_Draw_FindPic (char *name);
+struct image_s	*RE_Draw_FindPic (const char *name);
 
-void	RE_Draw_GetPicSize (int *w, int *h, char *name);
-void	RE_Draw_PicScaled (int x, int y, char *name, float scale);
-void	RE_Draw_StretchPic (int x, int y, int w, int h, char *name);
+void	RE_Draw_GetPicSize (int *w, int *h, const char *name);
+void	RE_Draw_PicScaled (int x, int y, const char *name, float scale);
+void	RE_Draw_StretchPic (int x, int y, int w, int h, const char *name);
 void	RE_Draw_StretchRaw (int x, int y, int w, int h, int cols, int rows, const byte *data, int bits);
 void	RE_Draw_CharScaled (int x, int y, int c, float scale);
-void	RE_Draw_TileClear (int x, int y, int w, int h, char *name);
+void	RE_Draw_TileClear (int x, int y, int w, int h, const char *name);
 void	RE_Draw_Fill (int x, int y, int w, int h, int c);
 void	RE_Draw_FadeScreen (void);
 
