@@ -64,7 +64,7 @@ enum GameType {
 	rogue
 };
 
-struct {
+static struct {
 	qboolean saved;
 	int curfile;
 	int numsamples;
@@ -88,15 +88,24 @@ OGG_TogglePlayback(void);
 static int getMappedGOGtrack(int track, enum GameType gameType)
 {
 	if(track <= 0)
+	{
 		return 0;
+	}
 
 	if(track == 1)
+	{
 		return 0; // 1 is illegal (=> data track on CD), 0 means "no track"
+	}
 
 	if(gameType == other)
+	{
 		return track;
+	}
+
 	if(gameType == rogue)
+	{
 		return track + 10;
+	}
 
 	// apparently it's xatrix => map the track to the corresponding TrackXX.ogg from GOG
 	switch(track)
@@ -185,14 +194,16 @@ OGG_InitTrackList(void)
 
 			char testFileName[MAX_OSPATH];
 
-			// the simple case (like before: $mod/music/02.ogg - 11.ogg or whatever)
+			/* the simple case (like before: $mod/music/02.ogg - 11.ogg or whatever) */
 			snprintf(testFileName, MAX_OSPATH, "%s02.ogg", fullMusicPath);
 
 			if(Sys_IsFile(testFileName))
 			{
+				int i;
+
 				ogg_tracks[2] = strdup(testFileName);
 
-				for(int i=3; i<MAX_NUM_OGGTRACKS; ++i)
+				for(i = 3; i < MAX_NUM_OGGTRACKS; ++i)
 				{
 					snprintf(testFileName, MAX_OSPATH, "%s%02i.ogg", fullMusicPath, i);
 
@@ -206,13 +217,15 @@ OGG_InitTrackList(void)
 				return;
 			}
 
-			// the GOG case: music/Track02.ogg to Track21.ogg
+			/* the GOG case: music/Track02.ogg to Track21.ogg */
 			snprintf(testFileName, MAX_OSPATH, "%sTrack%02i.ogg",
 				fullMusicPath, getMappedGOGtrack(8, gameType));
 
 			if(Sys_IsFile(testFileName))
 			{
-				for(int i=2; i<MAX_NUM_OGGTRACKS; ++i)
+				int i;
+
+				for(i = 2; i < MAX_NUM_OGGTRACKS; ++i)
 				{
 					int gogTrack = getMappedGOGtrack(i, gameType);
 
@@ -227,10 +240,34 @@ OGG_InitTrackList(void)
 
 				return;
 			}
+
+			/* the ReRelease case: music/track02.ogg to track79.ogg */
+			snprintf(testFileName, MAX_OSPATH, "%strack%02i.ogg",
+				fullMusicPath, getMappedGOGtrack(8, gameType));
+
+			if(Sys_IsFile(testFileName))
+			{
+				int i;
+
+				for(i = 2; i < MAX_NUM_OGGTRACKS; ++i)
+				{
+					int gogTrack = getMappedGOGtrack(i, gameType);
+
+					snprintf(testFileName, MAX_OSPATH, "%strack%02i.ogg", fullMusicPath, gogTrack);
+
+					if(Sys_IsFile(testFileName))
+					{
+						ogg_tracks[i] = strdup(testFileName);
+						ogg_maxfileindex = i;
+					}
+				}
+
+				return;
+			}
 		}
 	}
 
-	// if tracks have been found above, we would've returned there
+	/* if tracks have been found above, we would've returned there */
 	Com_Printf("No Ogg Vorbis music tracks have been found, so there will be no music.\n");
 }
 
@@ -267,7 +304,7 @@ static OGG_Read(void)
 		ogg_numbufs = 0;
 		ogg_numsamples = 0;
 
-		OGG_PlayTrack(ogg_curfile, false, false);
+		OGG_PlayTrack(va("%d", ogg_curfile), false, false);
 	}
 }
 
@@ -357,7 +394,7 @@ OGG_Stream(void)
 
 		// Ogg playback has stopped and shuffle was modified, play the map cdtrack.
 		ogg_shuffle->modified = false;
-		OGG_PlayTrack(ogg_mapcdtrack, true, true);
+		OGG_PlayTrack(va("%d", ogg_mapcdtrack), true, true);
 	}
 }
 
@@ -367,8 +404,11 @@ OGG_Stream(void)
  * play the ogg file that corresponds to the CD track with the given number
  */
 void
-OGG_PlayTrack(int trackNo, qboolean cdtrack, qboolean immediate)
+OGG_PlayTrack(const char *track, qboolean cdtrack, qboolean immediate)
 {
+	char name[MAX_OSPATH], *path = NULL;
+	int trackNo;
+
 	if (sound_started == SS_NOT)
 	{
 		return; // sound is not initialized
@@ -379,6 +419,53 @@ OGG_PlayTrack(int trackNo, qboolean cdtrack, qboolean immediate)
 		return;
 	}
 
+	while (1)
+	{
+		int res = 0;
+		FILE* f;
+
+		path = FS_NextPath(path);
+
+		if (!path)
+		{
+			break;
+		}
+
+		Com_sprintf(name, sizeof(name), "%s/music/%s", path, track);
+
+		/* Open ogg vorbis file. */
+		f = Q_fopen(name, "rb");
+		if (f == NULL)
+		{
+			continue;
+		}
+
+		/* Check running music. */
+		if (ogg_status == PLAY)
+		{
+			OGG_Stop();
+		}
+
+		// fclose is not required on error with close_on_free=true
+		ogg_file = stb_vorbis_open_file(f, true, &res, NULL);
+
+		if (res != 0)
+		{
+			Com_Printf("%s: '%s' is not a valid Ogg Vorbis file (error %i).\n",
+				__func__, name, res);
+			return;
+		}
+
+		/* Play file. */
+		ogg_curfile = 0;
+		ogg_numsamples = 0;
+		ogg_status = PLAY;
+
+		return;
+	}
+
+	trackNo = (int)strtol(track, (char **)NULL, 10);
+
 	// Track 0 means "stop music".
 	if (trackNo == 0)
 	{
@@ -439,10 +526,11 @@ OGG_PlayTrack(int trackNo, qboolean cdtrack, qboolean immediate)
 		return;
 	} break;
 	case 2:			// sequential
-	{		
+	{
 		newtrack = (curtrack + 1) % (ogg_maxfileindex + 1) != 0 ? (curtrack + 1) : 2;
 	} break;
 	case 3:			// random
+	case 4:			// random with true randomness
 	{
 		int retries = 100;
 		newtrack = 0;
@@ -451,9 +539,12 @@ OGG_PlayTrack(int trackNo, qboolean cdtrack, qboolean immediate)
 		{
 			newtrack = randk() % (ogg_maxfileindex + 1);
 
-			if (newtrack == curtrack)
+			if (playback == 3)
 			{
-				newtrack = 0;
+				if (newtrack == curtrack)
+				{
+					newtrack = 0;
+				}
 			}
 		}
 	} break;
@@ -474,7 +565,8 @@ OGG_PlayTrack(int trackNo, qboolean cdtrack, qboolean immediate)
 
 	if (ogg_tracks[trackNo] == NULL)
 	{
-		Com_Printf("%s: Don't have a .ogg file for track %d\n", __func__, trackNo);
+		Com_Printf("%s: Don't have a .ogg file for track %d\n",
+			__func__, trackNo);
 	}
 
 	/* Check running music. */
@@ -492,7 +584,8 @@ OGG_PlayTrack(int trackNo, qboolean cdtrack, qboolean immediate)
 
 	if (ogg_tracks[trackNo] == NULL)
 	{
-		Com_Printf("OGG_PlayTrack: I don't have a file for track %d!\n", trackNo);
+		Com_Printf("%s: I don't have a file for track %d!\n",
+			__func__, trackNo);
 
 		return;
 	}
@@ -502,7 +595,8 @@ OGG_PlayTrack(int trackNo, qboolean cdtrack, qboolean immediate)
 
 	if (f == NULL)
 	{
-		Com_Printf("%s: could not open file %s for track %d: %s.\n", __func__, ogg_tracks[trackNo], trackNo, strerror(errno));
+		Com_Printf("%s: could not open file %s for track %d: %s.\n",
+			__func__, ogg_tracks[trackNo], trackNo, strerror(errno));
 		ogg_tracks[trackNo] = NULL;
 
 		return;
@@ -515,7 +609,8 @@ OGG_PlayTrack(int trackNo, qboolean cdtrack, qboolean immediate)
 
 	if (res != 0)
 	{
-		Com_Printf("%s: '%s' is not a valid Ogg Vorbis file (error %i).\n", __func__, ogg_tracks[trackNo], res);
+		Com_Printf("%s: '%s' is not a valid Ogg Vorbis file (error %i).\n",
+			__func__, ogg_tracks[trackNo], res);
 
 		return;
 	}
@@ -626,7 +721,7 @@ OGG_TogglePlayback(void)
 /*
  * Prints a help message for the 'ogg' cmd.
  */
-void
+static void
 OGG_HelpMsg(void)
 {
 	Com_Printf("Unknown sub command %s\n\n", Cmd_Argv(1));
@@ -641,7 +736,7 @@ OGG_HelpMsg(void)
 /*
  * The 'ogg' cmd. Gives some control and information about the playback state.
  */
-void
+static void
 OGG_Cmd(void)
 {
 	if (Cmd_Argc() < 2)
@@ -662,17 +757,7 @@ OGG_Cmd(void)
 			return;
 		}
 
-		int track = (int)strtol(Cmd_Argv(2), NULL, 10);
-
-		if (track < 2 || track > ogg_maxfileindex)
-		{
-			Com_Printf("invalid track %s, must be an number between 2 and %d\n", Cmd_Argv(1), ogg_maxfileindex);
-			return;
-		}
-		else
-		{
-			OGG_PlayTrack(track, false, true);
-		}
+		OGG_PlayTrack(Cmd_Argv(2), false, true);
 	}
 	else if (Q_stricmp(Cmd_Argv(1), "stop") == 0)
 	{
@@ -698,10 +783,9 @@ OGG_Cmd(void)
 void
 OGG_SaveState(void)
 {
-	if (ogg_status != PLAY)
+	if (ogg_enabled->value != 1 || ogg_status != PLAY)
 	{
 		ogg_saved_state.saved = false;
-
 		return;
 	}
 
@@ -716,7 +800,7 @@ OGG_SaveState(void)
 void
 OGG_RecoverState(void)
 {
-	if (!ogg_saved_state.saved)
+	if (ogg_enabled->value != 1 || ogg_saved_state.saved != true)
 	{
 		return;
 	}
@@ -727,7 +811,7 @@ OGG_RecoverState(void)
 	int shuffle_state = ogg_shuffle->value;
 	Cvar_SetValue("ogg_shuffle", 0);
 
-	OGG_PlayTrack(ogg_saved_state.curfile, false, true);
+	OGG_PlayTrack(va("%d", ogg_saved_state.curfile), false, true);
 	stb_vorbis_seek_frame(ogg_file, ogg_saved_state.numsamples);
 	ogg_numsamples = ogg_saved_state.numsamples;
 
