@@ -99,7 +99,7 @@ ClearDisplayIndices(void)
 }
 
 static qboolean
-CreateSDLWindow(int flags, int fullscreen, int w, int h)
+CreateSDLWindow(SDL_WindowFlags flags, int fullscreen, int w, int h)
 {
 	if (SDL_WINDOWPOS_ISUNDEFINED(last_position_x) || SDL_WINDOWPOS_ISUNDEFINED(last_position_y) || last_position_x < 0 ||last_position_y < 24)
 	{
@@ -121,13 +121,16 @@ CreateSDLWindow(int flags, int fullscreen, int w, int h)
 	SDL_SetNumberProperty(props, SDL_PROP_WINDOW_CREATE_Y_NUMBER, last_position_y);
 	SDL_SetNumberProperty(props, SDL_PROP_WINDOW_CREATE_WIDTH_NUMBER, w);
 	SDL_SetNumberProperty(props, SDL_PROP_WINDOW_CREATE_HEIGHT_NUMBER, h);
-	SDL_SetNumberProperty(props, "flags", flags);
+	SDL_SetNumberProperty(props, SDL_PROP_WINDOW_CREATE_FLAGS_NUMBER, flags);
 
 	window = SDL_CreateWindowWithProperties(props);
 	SDL_DestroyProperties(props);
 
 	if (window)
 	{
+		/* enable text input */
+		SDL_StartTextInput(window);
+
 		/* save current display as default */
 		if ((last_display = SDL_GetDisplayForWindow(window)) == 0)
 		{
@@ -153,19 +156,17 @@ CreateSDLWindow(int flags, int fullscreen, int w, int h)
 
 			/* Otherwise try to find a mode near the requested one and
 			   switch to it in exclusive fullscreen mode. */
-			/* TODO SDL3: Leak? */
-		    const SDL_DisplayMode *closestMode = SDL_GetClosestFullscreenDisplayMode(last_display, w, h, vid_rate->value, false);
+			SDL_DisplayMode closestMode;
 
-			if (closestMode == NULL)
+			if (SDL_GetClosestFullscreenDisplayMode(last_display, w, h, vid_rate->value, false, &closestMode) != true)
 			{
 				Com_Printf("SDL was unable to find a mode close to %ix%i@%f\n", w, h, vid_rate->value);
 
 				if (vid_rate->value != 0)
 				{
 					Com_Printf("Retrying with desktop refresh rate\n");
-					closestMode = SDL_GetClosestFullscreenDisplayMode(last_display, w, h, 0, false);
 
-					if (closestMode != NULL)
+					if (SDL_GetClosestFullscreenDisplayMode(last_display, w, h, vid_rate->value, false, &closestMode) == true)
 					{
 						Cvar_SetValue("vid_rate", 0);
 					}
@@ -178,31 +179,24 @@ CreateSDLWindow(int flags, int fullscreen, int w, int h)
 			}
 
 			Com_Printf("User requested %ix%i@%f, setting closest mode %ix%i@%f\n",
-					w, h, vid_rate->value, closestMode->w, closestMode->h , closestMode->refresh_rate);
+					w, h, vid_rate->value, closestMode.w, closestMode.h , closestMode.refresh_rate);
 
 
 			/* TODO SDL3: Same code is in InitGraphics(), refactor into
 			 * a function? */
-			if (SDL_SetWindowFullscreenMode(window, closestMode) < 0)
+			if (!SDL_SetWindowFullscreenMode(window, &closestMode))
 			{
 				Com_Printf("Couldn't set closest mode: %s\n", SDL_GetError());
 				return false;
 			}
 
-			if (SDL_SetWindowFullscreen(window, true) < 0)
+			if (!SDL_SetWindowFullscreen(window, true))
 			{
 				Com_Printf("Couldn't switch to exclusive fullscreen: %s\n", SDL_GetError());
 				return false;
 			}
 
-			int ret = SDL_SyncWindow(window);
-
-			if (ret > 0)
-			{
-				Com_Printf("Synchronizing window state timed out\n");
-				return false;
-			}
-			else if (ret < 0)
+			if (!SDL_SyncWindow(window))
 			{
 				Com_Printf("Couldn't synchronize window state: %s\n", SDL_GetError());
 				return false;
@@ -247,7 +241,7 @@ GetWindowSize(int* w, int* h)
 		return false;
 	}
 
-	if (SDL_GetWindowSize(window, w, h) < 0)
+	if (!SDL_GetWindowSize(window, w, h))
 	{
 		Com_Printf("Couldn't get window size: %s\n", SDL_GetError());
 		return false;
@@ -304,7 +298,7 @@ PrintDisplayModes(void)
 	}
 
 	int nummodes = 0;
-	const SDL_DisplayMode **modes = SDL_GetFullscreenDisplayModes(curdisplay, &nummodes);
+	SDL_DisplayMode **modes = SDL_GetFullscreenDisplayModes(curdisplay, &nummodes);
 
 	if (modes)
 	{
@@ -347,7 +341,7 @@ SetSDLIcon()
 	amask = (q2icon64.bytes_per_pixel == 3) ? 0 : 0xff000000;
 #endif
 
-	SDL_Surface* icon = SDL_CreateSurfaceFrom((void *)q2icon64.pixel_data, q2icon64.width, q2icon64.height, q2icon64.bytes_per_pixel * q2icon64.width, SDL_GetPixelFormatEnumForMasks(q2icon64.bytes_per_pixel * 8, rmask, gmask, bmask, amask));
+	SDL_Surface* icon = SDL_CreateSurfaceFrom(q2icon64.width, q2icon64.height, SDL_GetPixelFormatForMasks(q2icon64.bytes_per_pixel * 8, rmask, gmask, bmask, amask), (void *)q2icon64.pixel_data, q2icon64.bytes_per_pixel * q2icon64.width);
 	SDL_SetWindowIcon(window, icon);
 	SDL_DestroySurface(icon);
 }
@@ -407,18 +401,17 @@ GLimp_Init(void)
 
 	if (!SDL_WasInit(SDL_INIT_VIDEO))
 	{
-		if (SDL_Init(SDL_INIT_VIDEO) == -1)
+		if (!SDL_Init(SDL_INIT_VIDEO))
 		{
 			Com_Printf("Couldn't init SDL video: %s.\n", SDL_GetError());
 
 			return false;
 		}
 
-		SDL_Version version;
+		int version = SDL_GetVersion();
 
-		SDL_GetVersion(&version);
 		Com_Printf("-------- vid initialization --------\n");
-		Com_Printf("SDL version is: %i.%i.%i\n", (int)version.major, (int)version.minor, (int)version.patch);
+		Com_Printf("SDL version is: %i.%i.%i\n", SDL_VERSIONNUM_MAJOR(version), SDL_VERSIONNUM_MINOR(version), SDL_VERSIONNUM_MICRO(version));
 		Com_Printf("SDL video driver is \"%s\".\n", SDL_GetCurrentVideoDriver());
 
 		SDL_DisplayID *displays;
@@ -461,7 +454,7 @@ GLimp_Shutdown(void)
  * compositor might scale us.
  */
 static int
-Glimp_DetermineHighDPISupport(int flags)
+Glimp_DetermineHighDPISupport(SDL_WindowFlags flags)
 {
 	/* Make sure that high dpi is never set when we don't want it. */
 	flags &= ~SDL_WINDOW_HIGH_PIXEL_DENSITY;
@@ -488,11 +481,11 @@ Glimp_DetermineHighDPISupport(int flags)
 qboolean
 GLimp_InitGraphics(int fullscreen, int *pwidth, int *pheight)
 {
-	int flags;
+	SDL_WindowFlags flags;
+	SDL_WindowFlags fs_flag = 0;
 	int curWidth, curHeight;
 	int width = *pwidth;
 	int height = *pheight;
-	unsigned int fs_flag = 0;
 
 	if (fullscreen == FULLSCREEN_EXCLUSIVE || fullscreen == FULLSCREEN_DESKTOP)
 	{
@@ -506,26 +499,22 @@ GLimp_InitGraphics(int fullscreen, int *pwidth, int *pheight)
 	if (initSuccessful && GetWindowSize(&curWidth, &curHeight)
 			&& (curWidth == width) && (curHeight == height))
 	{
-		/* TODO SDL3: Leak? */
-		const SDL_DisplayMode *closestMode = NULL;
+		SDL_DisplayMode closestMode;
 
 		/* If we want fullscreen, but aren't */
 		if (GetFullscreenType())
 		{
 			if (fullscreen == FULLSCREEN_EXCLUSIVE)
 			{
-				closestMode = SDL_GetClosestFullscreenDisplayMode(last_display, width, height, vid_rate->value, false);
-
-				if (closestMode == NULL)
+				if (SDL_GetClosestFullscreenDisplayMode(last_display, width, height, vid_rate->value, false, &closestMode) != true)
 				{
 					Com_Printf("SDL was unable to find a mode close to %ix%i@%f\n", width, height, vid_rate->value);
 
 					if (vid_rate->value != 0)
 					{
 						Com_Printf("Retrying with desktop refresh rate\n");
-						closestMode = SDL_GetClosestFullscreenDisplayMode(last_display, width, height, 0, false);
 
-						if (closestMode != NULL)
+						if (SDL_GetClosestFullscreenDisplayMode(last_display, width, height, 0, false, &closestMode) == true)
 						{
 							Cvar_SetValue("vid_rate", 0);
 						}
@@ -540,31 +529,24 @@ GLimp_InitGraphics(int fullscreen, int *pwidth, int *pheight)
 			else if (fullscreen == FULLSCREEN_DESKTOP)
 			{
 				/* Fullscreen window */
-				closestMode = NULL;
+				/* closestMode = NULL; */
 			}
 
-			if (SDL_SetWindowFullscreenMode(window, closestMode) < 0)
+			if (!SDL_SetWindowFullscreenMode(window, &closestMode))
 			{
 				Com_Printf("Couldn't set fullscreen modmode: %s\n", SDL_GetError());
 				Cvar_SetValue("vid_fullscreen", 0);
 			}
 			else
 			{
-				if (SDL_SetWindowFullscreen(window, true) < 0)
+				if (!SDL_SetWindowFullscreen(window, true))
 				{
 					Com_Printf("Couldn't switch to exclusive fullscreen: %s\n", SDL_GetError());
 					Cvar_SetValue("vid_fullscreen", 0);
 				}
 				else
 				{
-					int ret = SDL_SyncWindow(window);
-
-					if (ret > 0)
-					{
-						Com_Printf("Synchronizing window state timed out\n");
-						Cvar_SetValue("vid_fullscreen", 0);
-					}
-					else if (ret < 0)
+					if (!SDL_SyncWindow(window))
 					{
 						Com_Printf("Couldn't synchronize window state: %s\n", SDL_GetError());
 						Cvar_SetValue("vid_fullscreen", 0);
@@ -779,13 +761,12 @@ GLimp_GrabInput(qboolean grab)
 {
 	if(window != NULL)
 	{
-		SDL_SetWindowMouseGrab(window, grab ? SDL_TRUE : SDL_FALSE);
+		SDL_SetWindowMouseGrab(window, grab ? true : false);
 	}
 
-	if(SDL_SetRelativeMouseMode(grab ? SDL_TRUE : SDL_FALSE) < 0)
+	if(!SDL_SetWindowRelativeMouseMode(window, grab ? true : false))
 	{
 		Com_Printf("WARNING: Setting Relative Mousemode failed, reason: %s\n", SDL_GetError());
-		Com_Printf("         You should probably update to SDL 2.0.3 or newer!\n");
 	}
 }
 
@@ -906,8 +887,6 @@ GLimp_GetWindowDisplayIndex(void)
 int
 GLimp_GetFrameworkVersion(void)
 {
-	SDL_Version ver;
-	SDL_VERSION(&ver);
-
-	return ver.major;
+	int version = SDL_GetVersion();
+	return SDL_VERSIONNUM_MAJOR(version);
 }
