@@ -105,7 +105,7 @@ void VID_WriteScreenshot(int width, int height, int comp, const void* data)
 		if (argc > 2)
 		{
 			const char* q = Cmd_Argv(2);
-			int qualityStrLen = strlen(q);
+			size_t qualityStrLen = strlen(q);
 
 			for (i = 0; i < qualityStrLen; ++i)
 			{
@@ -240,6 +240,7 @@ vidmode_t vid_modes[] = {
 	{"Mode 29: 3840x2160", 3840, 2160, 29},
 	{"Mode 30: 4096x2160", 4096, 2160, 30},
 	{"Mode 31: 5120x2880", 5120, 2880, 31},
+	{"Mode 32: 1600x900", 1600, 900, 32},
 };
 
 #define VID_NUM_MODES (sizeof(vid_modes) / sizeof(vid_modes[0]))
@@ -247,7 +248,7 @@ vidmode_t vid_modes[] = {
 /*
  * Callback function for the 'vid_listmodes' cmd.
  */
-void
+static void
 VID_ListModes_f(void)
 {
 	int i;
@@ -264,7 +265,7 @@ VID_ListModes_f(void)
 /*
  * Returns informations about the given mode.
  */
-qboolean
+static qboolean
 VID_GetModeInfo(int *width, int *height, int mode)
 {
 	if ((mode < 0) || (mode >= VID_NUM_MODES))
@@ -341,7 +342,7 @@ VID_HasRenderer(const char *renderer)
 /*
  * Called by the renderer to request a restart.
  */
-void
+static void
 VID_RequestRestart(ref_restart_t rs)
 {
 	restart_state = rs;
@@ -350,7 +351,7 @@ VID_RequestRestart(ref_restart_t rs)
 /*
  * Restarts the renderer.
  */
-void
+static void
 VID_Restart_f(void)
 {
 	if (restart_state == RESTART_UNDEF)
@@ -364,7 +365,7 @@ VID_Restart_f(void)
 /*
  * Shuts the renderer down and unloads it.
  */
-void
+static void
 VID_ShutdownRenderer(void)
 {
 	if (ref_active)
@@ -384,7 +385,7 @@ VID_ShutdownRenderer(void)
 /*
  * Loads and initializes a renderer.
  */
-qboolean
+static qboolean
 VID_LoadRenderer(void)
 {
 	refimport_t	ri;
@@ -532,10 +533,11 @@ VID_CheckChanges(void)
 		// Mkay, let's try our luck.
 		while (!VID_LoadRenderer())
 		{
-			// We try: custom -> gl3 -> gles3 -> gl1 -> soft.
+			// We try: custom -> gl3 -> gles3 -> gl1 -> gles1 -> soft.
 			if ((strcmp(vid_renderer->string, "gl3") != 0) &&
 				(strcmp(vid_renderer->string, "gles3") != 0) &&
 				(strcmp(vid_renderer->string, "gl1") != 0) &&
+				(strcmp(vid_renderer->string, "gles1") != 0) &&
 				(strcmp(vid_renderer->string, "soft") != 0))
 			{
 				Com_Printf("Retrying with gl3...\n");
@@ -552,6 +554,11 @@ VID_CheckChanges(void)
 				Cvar_Set("vid_renderer", "gl1");
 			}
 			else if (strcmp(vid_renderer->string, "gl1") == 0)
+			{
+				Com_Printf("Retrying with gles1...\n");
+				Cvar_Set("vid_renderer", "gles1");
+			}
+			else if (strcmp(vid_renderer->string, "gles1") == 0)
 			{
 				Com_Printf("Retrying with soft...\n");
 				Cvar_Set("vid_renderer", "soft");
@@ -636,7 +643,7 @@ R_RegisterModel(char *name)
 }
 
 struct image_s*
-R_RegisterSkin(char *name)
+R_RegisterSkin(const char *name)
 {
 	if (ref_active)
 	{
@@ -647,7 +654,7 @@ R_RegisterSkin(char *name)
 }
 
 void
-R_SetSky(char *name, float rotate, vec3_t axis)
+R_SetSky(const char *name, float rotate, vec3_t axis)
 {
 	if (ref_active)
 	{
@@ -674,7 +681,7 @@ R_RenderFrame(refdef_t *fd)
 }
 
 struct image_s*
-Draw_FindPic(char *name)
+Draw_FindPic(const char *name)
 {
 	if (ref_active)
 	{
@@ -686,7 +693,7 @@ Draw_FindPic(char *name)
 
 
 void
-Draw_GetPicSize(int *w, int *h, char *name)
+Draw_GetPicSize(int *w, int *h, const char *name)
 {
 	if (ref_active)
 	{
@@ -695,7 +702,7 @@ Draw_GetPicSize(int *w, int *h, char *name)
 }
 
 void
-Draw_StretchPic(int x, int y, int w, int h, char *name)
+Draw_StretchPic(int x, int y, int w, int h, const char *name)
 {
 	if (ref_active)
 	{
@@ -704,7 +711,7 @@ Draw_StretchPic(int x, int y, int w, int h, char *name)
 }
 
 void
-Draw_PicScaled(int x, int y, char *pic, float factor)
+Draw_PicScaled(int x, int y, const char *pic, float factor)
 {
 	if (ref_active)
 	{
@@ -722,7 +729,7 @@ Draw_CharScaled(int x, int y, int num, float scale)
 }
 
 void
-Draw_TileClear(int x, int y, int w, int h, char *name)
+Draw_TileClear(int x, int y, int w, int h, const char *name)
 {
 	if (ref_active)
 	{
