@@ -32,7 +32,7 @@
 #include "shared.h"
 #include "crc.h"
 
-#define YQ2VERSION "8.41"
+#define YQ2VERSION "8.50"
 #define BASEDIRNAME "baseq2"
 
 #ifndef YQ2OSTYPE
@@ -448,13 +448,13 @@ void Cmd_ForwardToServer(void);
 
 extern cvar_t *cvar_vars;
 
-cvar_t *Cvar_Get(const char *var_name, char *value, int flags);
+cvar_t *Cvar_Get(const char *var_name, const char *value, int flags);
 
 /* creates the variable if it doesn't exist, or returns the existing one */
 /* if it exists, the value will not be changed, but flags will be ORed in */
 /* that allows variables to be unarchived without needing bitflags */
 
-cvar_t *Cvar_Set(const char *var_name, char *value);
+cvar_t *Cvar_Set(const char *var_name, const char *value);
 
 /* will create the variable if it doesn't exist */
 
@@ -462,7 +462,7 @@ cvar_t *Cvar_ForceSet(const char *var_name, char *value);
 
 /* will set the variable even if NOSET or LATCH */
 
-cvar_t *Cvar_FullSet(const char *var_name, char *value, int flags);
+cvar_t *Cvar_FullSet(const char *var_name, const char *value, int flags);
 
 void Cvar_SetValue(const char *var_name, float value);
 
@@ -616,7 +616,7 @@ qboolean Netchan_CanReliable(netchan_t *chan);
 #include "files.h"
 
 cmodel_t *CM_LoadMap(char *name, qboolean clientload, unsigned *checksum);
-cmodel_t *CM_InlineModel(char *name);       /* *1, *2, etc */
+cmodel_t *CM_InlineModel(const char *name);       /* *1, *2, etc */
 
 int CM_NumClusters(void);
 int CM_NumInlineModels(void);
@@ -701,18 +701,18 @@ int FS_FRead(void *buffer, int size, int count, fileHandle_t f);
 // returns NULL if f is no valid handle
 const char* FS_GetFilenameForHandle(fileHandle_t f);
 
-char **FS_ListFiles(char *findname, int *numfiles,
+char **FS_ListFiles(const char *findname, int *numfiles,
 		unsigned musthave, unsigned canthave);
-char **FS_ListFiles2(char *findname, int *numfiles,
+char **FS_ListFiles2(const char *findname, int *numfiles,
 		unsigned musthave, unsigned canthave);
 void FS_FreeList(char **list, int nfiles);
 
 void FS_InitFilesystem(void);
 void FS_ShutdownFilesystem(void);
-void FS_BuildGameSpecificSearchPath(char *dir);
+void FS_BuildGameSpecificSearchPath(const char *dir);
 char *FS_Gamedir(void);
-char *FS_NextPath(char *prevpath);
-int FS_LoadFile(char *path, void **buffer);
+char *FS_NextPath(const char *prevpath);
+int FS_LoadFile(const char *path, void **buffer);
 qboolean FS_FileInGamedir(const char *file);
 qboolean FS_AddPAKFromGamedir(const char *pak);
 const char* FS_GetNextRawPath(const char* lastRawPath);
@@ -868,4 +868,6 @@ void Sys_SetupFPU(void);
 
 /* ======================================================================= */
 
+void Mods_NamesFinish(void);
+
 #endif
