@@ -28,8 +28,6 @@
 #define CO_CRC_H
 
 void CRC_Init(unsigned short *crcvalue);
-void CRC_ProcessByte(unsigned short *crcvalue, byte data);
-unsigned short CRC_Value(unsigned short crcvalue);
 unsigned short CRC_Block(byte *start, int count);
 
 #endif
