@@ -45,13 +45,13 @@ void Weapon_GrenadeLauncher(edict_t *ent);
 void Weapon_Railgun(edict_t *ent);
 void Weapon_BFG(edict_t *ent);
 
-gitem_armor_t jacketarmor_info = {25, 50, .30, .00, ARMOR_JACKET};
-gitem_armor_t combatarmor_info = {50, 100, .60, .30, ARMOR_COMBAT};
-gitem_armor_t bodyarmor_info = {100, 200, .80, .60, ARMOR_BODY};
+static gitem_armor_t jacketarmor_info = {25, 50, .30, .00, ARMOR_JACKET};
+static gitem_armor_t combatarmor_info = {50, 100, .60, .30, ARMOR_COMBAT};
+static gitem_armor_t bodyarmor_info = {100, 200, .80, .60, ARMOR_BODY};
 
-int jacket_armor_index;
-int combat_armor_index;
-int body_armor_index;
+static int jacket_armor_index;
+static int combat_armor_index;
+static int body_armor_index;
 static int power_screen_index;
 static int power_shield_index;
 
@@ -196,8 +196,8 @@ Pickup_Powerup(edict_t *ent, edict_t *other)
 
 	quantity = other->client->pers.inventory[ITEM_INDEX(ent->item)];
 
-	if (((skill->value == 1) &&
-		 (quantity >= 2)) || ((skill->value >= 2) && (quantity >= 1)))
+	if (((skill->value == SKILL_MEDIUM) &&
+		 (quantity >= 2)) || ((skill->value >= SKILL_HARD) && (quantity >= 1)))
 	{
 		return false;
 	}
@@ -215,20 +215,6 @@ Pickup_Powerup(edict_t *ent, edict_t *other)
 		{
 			SetRespawn(ent, ent->item->quantity);
 		}
-
-		if (((int)dmflags->value & DF_INSTANT_ITEMS) ||
-			((ent->item->use == Use_Quad) &&
-			 (ent->spawnflags & DROPPED_PLAYER_ITEM)))
-		{
-			if ((ent->item->use == Use_Quad) &&
-				(ent->spawnflags & DROPPED_PLAYER_ITEM))
-			{
-				quad_drop_timeout_hack =
-					(ent->nextthink - level.time) / FRAMETIME;
-			}
-
-			ent->item->use(other, ent->item);
-		}
 	}
 
 	return true;
@@ -1214,6 +1200,29 @@ Touch_Item(edict_t *ent, edict_t *other, cplane_t *plane /* unused */, csurface_
 			gi.sound(other, CHAN_ITEM, gi.soundindex(
 							ent->item->pickup_sound), 1, ATTN_NORM, 0);
 		}
+
+		/* activate item instantly if appropriate */
+		/* moved down here so activation sounds override the pickup sound */
+		if (deathmatch->value)
+		{
+			if ((((int)dmflags->value & DF_INSTANT_ITEMS) &&
+				 (ent->item->flags & IT_INSTANT_USE)) ||
+				((ent->item->use == Use_Quad) &&
+				 (ent->spawnflags & DROPPED_PLAYER_ITEM)))
+			{
+				if ((ent->item->use == Use_Quad) &&
+					(ent->spawnflags & DROPPED_PLAYER_ITEM))
+				{
+					quad_drop_timeout_hack =
+						(ent->nextthink - level.time) / FRAMETIME;
+				}
+
+				if (ent->item->use)
+				{
+					ent->item->use(other, ent->item);
+				}
+			}
+		}
 	}
 
 	if (!(ent->spawnflags & ITEM_TARGETS_USED))
@@ -1301,13 +1310,13 @@ Drop_Item(edict_t *ent, gitem_t *item)
 	dropped->s.effects = item->world_model_flags;
 	dropped->s.renderfx = RF_GLOW;
 
-	if (randk() > 0.5)
+	if (frandk() > 0.5)
 	{
-		dropped->s.angles[1] += randk()*45;
+		dropped->s.angles[1] += frandk()*45;
 	}
 	else
 	{
-		dropped->s.angles[1] -= randk()*45;
+		dropped->s.angles[1] -= frandk()*45;
 	}
 
 	VectorSet (dropped->mins, -16, -16, -16);
@@ -1629,7 +1638,7 @@ SpawnItem(edict_t *ent, gitem_t *item)
 		}
 	}
 
-	if (coop->value && (strcmp(ent->classname, "key_power_cube") == 0))
+	if (coop->value && !(ent->spawnflags & ITEM_NO_TOUCH) && (strcmp(ent->classname, "key_power_cube") == 0))
 	{
 		ent->spawnflags |= (1 << (8 + level.power_cubes));
 		level.power_cubes++;
@@ -1655,7 +1664,7 @@ SpawnItem(edict_t *ent, gitem_t *item)
 
 /* ====================================================================== */
 
-gitem_t itemlist[] = {
+static const gitem_t gameitemlist[] = {
 	{
 		NULL
 	}, /* leave index 0 alone */
@@ -2160,7 +2169,7 @@ gitem_t itemlist[] = {
 		2,
 		60,
 		NULL,
-		IT_POWERUP,
+		IT_POWERUP | IT_INSTANT_USE,
 		0,
 		NULL,
 		0,
@@ -2182,7 +2191,7 @@ gitem_t itemlist[] = {
 		2,
 		300,
 		NULL,
-		IT_POWERUP,
+		IT_POWERUP | IT_INSTANT_USE,
 		0,
 		NULL,
 		0,
@@ -2204,7 +2213,7 @@ gitem_t itemlist[] = {
 		2,
 		60,
 		NULL,
-		IT_POWERUP,
+		IT_POWERUP | IT_INSTANT_USE,
 		0,
 		NULL,
 		0,
@@ -2226,7 +2235,7 @@ gitem_t itemlist[] = {
 		2,
 		60,
 		NULL,
-		IT_STAY_COOP | IT_POWERUP,
+		IT_STAY_COOP | IT_POWERUP | IT_INSTANT_USE,
 		0,
 		NULL,
 		0,
@@ -2248,7 +2257,7 @@ gitem_t itemlist[] = {
 		2,
 		60,
 		NULL,
-		IT_STAY_COOP | IT_POWERUP,
+		IT_STAY_COOP | IT_POWERUP | IT_INSTANT_USE,
 		0,
 		NULL,
 		0,
@@ -2576,6 +2585,8 @@ gitem_t itemlist[] = {
 	{NULL}
 };
 
+gitem_t itemlist[MAX_ITEMS];
+
 /*
  * QUAKED item_health (.3 .3 1) (-16 -16 -16) (16 16 16)
  */
@@ -2673,7 +2684,9 @@ SP_item_health_mega(edict_t *self)
 void
 InitItems(void)
 {
-	game.num_items = sizeof(itemlist) / sizeof(itemlist[0]) - 1;
+	memset(itemlist, 0, sizeof(itemlist));
+	memcpy(itemlist, gameitemlist, sizeof(gameitemlist));
+	game.num_items = sizeof(gameitemlist) / sizeof(gameitemlist[0]) - 1;
 }
 
 /*
