@@ -139,6 +139,7 @@ void SP_monster_mutant(edict_t *self);
 void SP_monster_supertank(edict_t *self);
 void SP_monster_boss2(edict_t *self);
 void SP_monster_jorg(edict_t *self);
+void SP_monster_makron(edict_t *self);
 void SP_monster_boss3_stand(edict_t *self);
 
 void SP_monster_commander_body(edict_t *self);
@@ -147,7 +148,7 @@ void SP_turret_breach(edict_t *self);
 void SP_turret_base(edict_t *self);
 void SP_turret_driver(edict_t *self);
 
-spawn_t spawns[] = {
+static spawn_t spawns[] = {
 	{"item_health", SP_item_health},
 	{"item_health_small", SP_item_health_small},
 	{"item_health_large", SP_item_health_large},
@@ -257,6 +258,7 @@ spawn_t spawns[] = {
 	{"monster_supertank", SP_monster_supertank},
 	{"monster_boss2", SP_monster_boss2},
 	{"monster_boss3_stand", SP_monster_boss3_stand},
+	{"monster_makron", SP_monster_makron},
 	{"monster_jorg", SP_monster_jorg},
 
 	{"monster_commander_body", SP_monster_commander_body},
@@ -325,7 +327,12 @@ char *
 ED_NewString(const char *string)
 {
 	char *newb, *new_p;
-	int i, l;
+	size_t i, l;
+
+	if (!string)
+	{
+		return NULL;
+	}
 
 	l = strlen(string) + 1;
 
@@ -376,7 +383,7 @@ ED_ParseField(const char *key, const char *value, edict_t *ent)
 
 	for (f = fields; f->name; f++)
 	{
-		if (!(f->flags & FFL_NOSPAWN) && !Q_stricmp(f->name, key))
+		if (!(f->flags & FFL_NOSPAWN) && !Q_strcasecmp(f->name, (char *)key))
 		{
 			/* found it */
 			if (f->flags & FFL_SPAWNTEMP)
@@ -403,10 +410,10 @@ ED_ParseField(const char *key, const char *value, edict_t *ent)
 					*(int *)(b + f->ofs) = (int)strtol(value, (char **)NULL, 10);
 					break;
 				case F_FLOAT:
-					*(float *)(b + f->ofs) = strtod(value, (char **)NULL);
+					*(float *)(b + f->ofs) = (float)strtod(value, (char **)NULL);
 					break;
 				case F_ANGLEHACK:
-					v = strtod(value, (char **)NULL);
+					v = (float)strtod(value, (char **)NULL);
 					((float *)(b + f->ofs))[0] = 0;
 					((float *)(b + f->ofs))[1] = v;
 					((float *)(b + f->ofs))[2] = 0;
@@ -436,6 +443,11 @@ ED_ParseEdict(char *data, edict_t *ent)
 	char keyname[256];
 	const char *com_token;
 
+	if (!ent)
+	{
+		return NULL;
+	}
+
 	init = false;
 	memset(&st, 0, sizeof(st));
 
@@ -572,7 +584,6 @@ SpawnEntities(const char *mapname, char *entities, const char *spawnpoint)
 	const char *com_token;
 	int i;
 	float skill_level;
-	static qboolean monster_count_city3 = false;
 
 	if (!mapname || !entities || !spawnpoint)
 	{
@@ -650,21 +661,6 @@ SpawnEntities(const char *mapname, char *entities, const char *spawnpoint)
 			ent->spawnflags &= ~SPAWNFLAG_NOT_HARD;
 		}
 
-		/*
-		 * The 'monsters' count in city3.bsp is wrong.
-		 * There're two monsters triggered in a hidden
-		 * and unreachable room next to the security
-		 * pass.
-		 *
-		 * We need to make sure that this hack is only
-		 * applied once!
-		 */
-		if(!Q_stricmp(level.mapname, "city3") && !monster_count_city3)
-		{
-			level.total_monsters = level.total_monsters - 2;
-			monster_count_city3 = true;
-		}
-
 		/* remove things (except the world) from
 		   different skill levels or deathmatch */
 		if (ent != g_edicts)
@@ -680,12 +676,12 @@ SpawnEntities(const char *mapname, char *entities, const char *spawnpoint)
 			}
 			else
 			{
-				if (((skill->value == 0) &&
+				if (((skill->value == SKILL_EASY) &&
 					 (ent->spawnflags & SPAWNFLAG_NOT_EASY)) ||
-					((skill->value == 1) &&
+					((skill->value == SKILL_MEDIUM) &&
 					 (ent->spawnflags & SPAWNFLAG_NOT_MEDIUM)) ||
-					(((skill->value == 2) ||
-					  (skill->value == 3)) &&
+					(((skill->value == SKILL_HARD) ||
+					  (skill->value == SKILL_HARDPLUS)) &&
 					 (ent->spawnflags & SPAWNFLAG_NOT_HARD))
 					)
 				{
@@ -704,6 +700,9 @@ SpawnEntities(const char *mapname, char *entities, const char *spawnpoint)
 		ED_CallSpawn(ent);
 	}
 
+	/* in case the last entity in the entstring has spawntemp fields */
+	memset(&st, 0, sizeof(st));
+
 	gi.dprintf("%i entities inhibited.\n", inhibit);
 
 	G_FindTeams();
@@ -713,7 +712,7 @@ SpawnEntities(const char *mapname, char *entities, const char *spawnpoint)
 
 /* =================================================================== */
 
-char *single_statusbar =
+static char *single_statusbar =
 	"yb	-24 "
 
 /* health */
@@ -771,7 +770,7 @@ char *single_statusbar =
 	"endif "
 ;
 
-char *dm_statusbar =
+static char *dm_statusbar =
 	"yb	-24 "
 
 /* health */
@@ -912,7 +911,14 @@ SP_worldspawn(edict_t *ent)
 	gi.configstring(CS_SKYAXIS, va("%f %f %f",
 				st.skyaxis[0], st.skyaxis[1], st.skyaxis[2]));
 
-	gi.configstring(CS_CDTRACK, va("%i", ent->sounds));
+	if (st.music && st.music[0])
+	{
+		gi.configstring(CS_CDTRACK, st.music);
+	}
+	else
+	{
+		gi.configstring(CS_CDTRACK, va("%i", ent->sounds));
+	}
 
 	gi.configstring(CS_MAXCLIENTS, va("%i", (int)(maxclients->value)));
 
@@ -977,20 +983,23 @@ SP_worldspawn(edict_t *ent)
 	gi.soundindex("*pain100_1.wav");
 	gi.soundindex("*pain100_2.wav");
 
-	/* sexed models you can add more, max 15
-	   THIS ORDER MUST MATCH THE DEFINES IN
-	   g_local.h  */
-	gi.modelindex("#w_blaster.md2");
-	gi.modelindex("#w_shotgun.md2");
-	gi.modelindex("#w_sshotgun.md2");
-	gi.modelindex("#w_machinegun.md2");
-	gi.modelindex("#w_chaingun.md2");
-	gi.modelindex("#a_grenades.md2");
-	gi.modelindex("#w_glauncher.md2");
-	gi.modelindex("#w_rlauncher.md2");
-	gi.modelindex("#w_hyperblaster.md2");
-	gi.modelindex("#w_railgun.md2");
-	gi.modelindex("#w_bfg.md2");
+	/* sexed models: THIS ORDER MUST MATCH THE DEFINES IN g_local.h
+	   you can add more, max 19 (pete change)these models are only
+	   loaded in coop or deathmatch. not singleplayer. */
+	if (coop->value || deathmatch->value)
+	{
+		gi.modelindex("#w_blaster.md2");
+		gi.modelindex("#w_shotgun.md2");
+		gi.modelindex("#w_sshotgun.md2");
+		gi.modelindex("#w_machinegun.md2");
+		gi.modelindex("#w_chaingun.md2");
+		gi.modelindex("#a_grenades.md2");
+		gi.modelindex("#w_glauncher.md2");
+		gi.modelindex("#w_rlauncher.md2");
+		gi.modelindex("#w_hyperblaster.md2");
+		gi.modelindex("#w_railgun.md2");
+		gi.modelindex("#w_bfg.md2");
+	}
 
 	/* ------------------- */
 
