@@ -27,11 +27,55 @@
 #include "../../header/local.h"
 #include "insane.h"
 
+#define SPAWNFLAG_CRUSIFIED		8
+
 static int sound_fist;
 static int sound_shake;
 static int sound_moan;
 static int sound_scream[8];
 
+static int  sound_step;
+static int  sound_step2;
+static int  sound_step3;
+static int  sound_step4;
+
+void
+insane_footstep(edict_t *self)
+{
+	if (!g_monsterfootsteps->value)
+		return;
+
+	// Lazy loading for savegame compatibility.
+	if (sound_step == 0 || sound_step2 == 0 || sound_step3 == 0 || sound_step4 == 0)
+	{
+		sound_step = gi.soundindex("player/step1.wav");
+		sound_step2 = gi.soundindex("player/step2.wav");
+		sound_step3 = gi.soundindex("player/step3.wav");
+		sound_step4 = gi.soundindex("player/step4.wav");
+	}
+
+	int     i;
+	i = randk() % 4;
+
+	if (i == 0)
+	{
+		gi.sound(self, CHAN_BODY, sound_step, 0.7, ATTN_NORM, 0);
+	}
+	else if (i == 1)
+	{
+		gi.sound(self, CHAN_BODY, sound_step2, 0.7, ATTN_NORM, 0);
+	}
+	else if (i == 2)
+	{
+		gi.sound(self, CHAN_BODY, sound_step3, 0.7, ATTN_NORM, 0);
+	}
+	else if (i == 3)
+	{
+		gi.sound(self, CHAN_BODY, sound_step4, 0.7, ATTN_NORM, 0);
+	}
+}
+
+
 void
 insane_fist(edict_t *self)
 {
@@ -62,6 +106,12 @@ insane_moan(edict_t *self)
 		return;
 	}
 
+	/* suppress screaming so pain sound can play */
+	if (self->fly_sound_debounce_time > level.time)
+	{
+		return;
+	}
+
 	gi.sound(self, CHAN_VOICE, sound_moan, 1, ATTN_IDLE, 0);
 }
 
@@ -73,6 +123,12 @@ insane_scream(edict_t *self)
 		return;
 	}
 
+	/* suppress screaming so pain sound can play */
+	if (self->fly_sound_debounce_time > level.time)
+	{
+		return;
+	}
+
 	gi.sound(self, CHAN_VOICE, sound_scream[randk() % 8], 1, ATTN_IDLE, 0);
 }
 
@@ -85,7 +141,7 @@ void insane_checkdown(edict_t *self);
 void insane_checkup(edict_t *self);
 void insane_onground(edict_t *self);
 
-mframe_t insane_frames_stand_normal[] = {
+static mframe_t insane_frames_stand_normal[] = {
 	{ai_stand, 0, NULL},
 	{ai_stand, 0, NULL},
 	{ai_stand, 0, NULL},
@@ -102,7 +158,7 @@ mmove_t insane_move_stand_normal =
    	insane_stand
 };
 
-mframe_t insane_frames_stand_insane[] = {
+static mframe_t insane_frames_stand_insane[] = {
 	{ai_stand, 0, insane_shake},
 	{ai_stand, 0, NULL},
 	{ai_stand, 0, NULL},
@@ -143,7 +199,7 @@ mmove_t insane_move_stand_insane =
    	insane_stand
 };
 
-mframe_t insane_frames_uptodown[] = {
+static mframe_t insane_frames_uptodown[] = {
 	{ai_move, 0, NULL},
 	{ai_move, 0, NULL},
 	{ai_move, 0, NULL},
@@ -197,7 +253,7 @@ mmove_t insane_move_uptodown =
    	insane_onground
 };
 
-mframe_t insane_frames_downtoup[] = {
+static mframe_t insane_frames_downtoup[] = {
 	{ai_move, -0.7, NULL}, /* 41 */
 	{ai_move, -1.2, NULL}, /* 42 */
 	{ai_move, -1.5, NULL}, /* 43 */
@@ -227,7 +283,7 @@ mmove_t insane_move_downtoup =
 	insane_stand
 };
 
-mframe_t insane_frames_jumpdown[] = {
+static mframe_t insane_frames_jumpdown[] = {
 	{ai_move, 0.2, NULL},
 	{ai_move, 11.5, NULL},
 	{ai_move, 5.1, NULL},
@@ -243,7 +299,7 @@ mmove_t insane_move_jumpdown =
    	insane_onground
 };
 
-mframe_t insane_frames_down[] = {
+static mframe_t insane_frames_down[] = {
 	{ai_move, 0, NULL}, /* 100 */
 	{ai_move, 0, NULL},
 	{ai_move, 0, NULL},
@@ -315,20 +371,20 @@ mmove_t insane_move_down =
    	insane_onground
 };
 
-mframe_t insane_frames_walk_normal[] = {
+static mframe_t insane_frames_walk_normal[] = {
 	{ai_walk, 0, insane_scream},
 	{ai_walk, 2.5, NULL},
 	{ai_walk, 3.5, NULL},
 	{ai_walk, 1.7, NULL},
 	{ai_walk, 2.3, NULL},
 	{ai_walk, 2.4, NULL},
-	{ai_walk, 2.2, NULL},
+	{ai_walk, 2.2, insane_footstep},
 	{ai_walk, 4.2, NULL},
 	{ai_walk, 5.6, NULL},
 	{ai_walk, 3.3, NULL},
 	{ai_walk, 2.4, NULL},
 	{ai_walk, 0.9, NULL},
-	{ai_walk, 0, NULL}
+	{ai_walk, 0, insane_footstep}
 };
 
 mmove_t insane_move_walk_normal =
@@ -346,32 +402,32 @@ mmove_t insane_move_run_normal =
 	insane_run
 };
 
-mframe_t insane_frames_walk_insane[] = {
+static mframe_t insane_frames_walk_insane[] = {
 	{ai_walk, 0, insane_scream}, /* walk 1 */
 	{ai_walk, 3.4, NULL}, /* walk 2 */
 	{ai_walk, 3.6, NULL}, /* 3 */
 	{ai_walk, 2.9, NULL}, /* 4 */
 	{ai_walk, 2.2, NULL}, /* 5 */
 	{ai_walk, 2.6, NULL}, /* 6 */
-	{ai_walk, 0, NULL}, /* 7 */
+	{ai_walk, 0, insane_footstep}, /* 7 */
 	{ai_walk, 0.7, NULL}, /* 8 */
 	{ai_walk, 4.8, NULL}, /* 9 */
 	{ai_walk, 5.3, NULL}, /* 10 */
 	{ai_walk, 1.1, NULL}, /* 11 */
-	{ai_walk, 2, NULL}, /* 12 */
+	{ai_walk, 2, insane_footstep}, /* 12 */
 	{ai_walk, 0.5, NULL}, /* 13 */
 	{ai_walk, 0, NULL}, /* 14 */
 	{ai_walk, 0, NULL}, /* 15 */
 	{ai_walk, 4.9, NULL}, /* 16 */
 	{ai_walk, 6.7, NULL}, /* 17 */
 	{ai_walk, 3.8, NULL}, /* 18 */
-	{ai_walk, 2, NULL}, /* 19 */
+	{ai_walk, 2, insane_footstep}, /* 19 */
 	{ai_walk, 0.2, NULL}, /* 20 */
 	{ai_walk, 0, NULL}, /* 21 */
 	{ai_walk, 3.4, NULL}, /* 22 */
 	{ai_walk, 6.4, NULL}, /* 23 */
 	{ai_walk, 5, NULL}, /* 24 */
-	{ai_walk, 1.8, NULL}, /* 25 */
+	{ai_walk, 1.8, insane_footstep}, /* 25 */
 	{ai_walk, 0, NULL} /* 26 */
 };
 
@@ -391,12 +447,12 @@ mmove_t insane_move_run_insane =
    	insane_run
 };
 
-mframe_t insane_frames_stand_pain[] = {
-	{ai_move, 0, NULL},
+static mframe_t insane_frames_stand_pain[] = {
 	{ai_move, 0, NULL},
 	{ai_move, 0, NULL},
 	{ai_move, 0, NULL},
 	{ai_move, 0, NULL},
+	{ai_move, 0, insane_footstep},
 	{ai_move, 0, NULL},
 	{ai_move, 0, NULL},
 	{ai_move, 0, NULL},
@@ -413,7 +469,7 @@ mmove_t insane_move_stand_pain =
    	insane_run
 };
 
-mframe_t insane_frames_stand_death[] = {
+static mframe_t insane_frames_stand_death[] = {
 	{ai_move, 0, NULL},
 	{ai_move, 0, NULL},
 	{ai_move, 0, NULL},
@@ -441,7 +497,7 @@ mmove_t insane_move_stand_death =
 	insane_dead
 };
 
-mframe_t insane_frames_crawl[] = {
+static mframe_t insane_frames_crawl[] = {
 	{ai_walk, 0, insane_scream},
 	{ai_walk, 1.5, NULL},
 	{ai_walk, 2.1, NULL},
@@ -469,7 +525,7 @@ mmove_t insane_move_runcrawl =
    	NULL
 };
 
-mframe_t insane_frames_crawl_pain[] = {
+static mframe_t insane_frames_crawl_pain[] = {
 	{ai_move, 0, NULL},
 	{ai_move, 0, NULL},
 	{ai_move, 0, NULL},
@@ -489,7 +545,7 @@ mmove_t insane_move_crawl_pain =
    	insane_run
 };
 
-mframe_t insane_frames_crawl_death[] = {
+static mframe_t insane_frames_crawl_death[] = {
 	{ai_move, 0, NULL},
 	{ai_move, 0, NULL},
 	{ai_move, 0, NULL},
@@ -507,7 +563,7 @@ mmove_t insane_move_crawl_death =
    	insane_dead
 };
 
-mframe_t insane_frames_cross[] = {
+static mframe_t insane_frames_cross[] = {
 	{ai_move, 0, insane_moan},
 	{ai_move, 0, NULL},
 	{ai_move, 0, NULL},
@@ -533,7 +589,7 @@ mmove_t insane_move_cross =
    	insane_cross
 };
 
-mframe_t insane_frames_struggle_cross[] = {
+static mframe_t insane_frames_struggle_cross[] = {
 	{ai_move, 0, insane_scream},
 	{ai_move, 0, NULL},
 	{ai_move, 0, NULL},
@@ -630,9 +686,7 @@ insane_run(edict_t *self)
 	{
 		self->monsterinfo.currentmove = &insane_move_runcrawl;
 	}
-	else
-
-	if (random() <= 0.5) /* Else, mix it up */
+	else if (frandk() <= 0.5) /* Else, mix it up */
 	{
 		self->monsterinfo.currentmove = &insane_move_run_normal;
 	}
@@ -682,13 +736,16 @@ insane_pain(edict_t *self, edict_t *other /* unused */,
 	gi.sound(self, CHAN_VOICE, gi.soundindex(va("player/male/pain%i_%i.wav",
 							l, r)), 1, ATTN_IDLE, 0);
 
-	if (skill->value == 3)
+	/* suppress screaming and moaning for 1 second so pain sound plays */
+	self->fly_sound_debounce_time = level.time + 1;
+
+	if (skill->value == SKILL_HARDPLUS)
 	{
 		return; /* no pain anims in nightmare */
 	}
 
 	/* Don't go into pain frames if crucified. */
-	if (self->spawnflags & 8)
+	if (self->spawnflags & SPAWNFLAG_CRUSIFIED)
 	{
 		self->monsterinfo.currentmove = &insane_move_struggle_cross;
 		return;
@@ -772,7 +829,7 @@ insane_stand(edict_t *self)
 		return;
 	}
 
-	if (self->spawnflags & 8) /* If crucified */
+	if (self->spawnflags & SPAWNFLAG_CRUSIFIED) /* If crucified */
 	{
 		self->monsterinfo.currentmove = &insane_move_cross;
 		self->monsterinfo.aiflags |= AI_STAND_GROUND;
@@ -801,7 +858,7 @@ insane_dead(edict_t *self)
 		return;
 	}
 
-	if (self->spawnflags & 8)
+	if (self->spawnflags & SPAWNFLAG_CRUSIFIED)
 	{
 		self->flags |= FL_FLY;
 	}
@@ -862,7 +919,7 @@ insane_die(edict_t *self, edict_t *inflictor /* unused */,
 	self->deadflag = DEAD_DEAD;
 	self->takedamage = DAMAGE_YES;
 
-	if (self->spawnflags & 8)
+	if (self->spawnflags & SPAWNFLAG_CRUSIFIED)
 	{
 		insane_dead(self);
 	}
@@ -899,6 +956,13 @@ SP_misc_insane(edict_t *self)
 		return;
 	}
 
+	// Force recaching at next footstep to ensure
+	// that the sound indices are correct.
+	sound_step = 0;
+	sound_step2 = 0;
+	sound_step3 = 0;
+	sound_step4 = 0;
+
 	sound_fist = gi.soundindex("insane/insane11.wav");
 	sound_shake = gi.soundindex("insane/insane5.wav");
 	sound_moan = gi.soundindex("insane/insane7.wav");
@@ -945,7 +1009,7 @@ SP_misc_insane(edict_t *self)
 
 	self->monsterinfo.scale = MODEL_SCALE;
 
-	if (self->spawnflags & 8) /* Crucified ? */
+	if (self->spawnflags & SPAWNFLAG_CRUSIFIED) /* Crucified ? */
 	{
 		VectorSet(self->mins, -16, 0, 0);
 		VectorSet(self->maxs, 16, 8, 32);
